@@ -31,7 +31,7 @@ from aesara.compile.ops import OutputGuard, _output_guard
 from aesara.configdefaults import config
 from aesara.graph.basic import Variable, io_toposort
 from aesara.graph.destroyhandler import DestroyHandler
-from aesara.graph.features import BadOptimization
+from aesara.graph.features import AlreadyThere, BadOptimization
 from aesara.graph.op import HasInnerGraph, Op
 from aesara.graph.utils import InconsistencyError, MethodNotDefined
 from aesara.link.basic import Container, LocalLinker
@@ -433,7 +433,9 @@ def _optcheck_fgraph(input_specs, output_specs, accept_inplace=False):
 
     """
     equivalence_tracker = _VariableEquivalenceTracker()
-    fgraph, updates = std_fgraph(input_specs, output_specs, accept_inplace)
+    fgraph, updates = std_fgraph(
+        input_specs, output_specs, accept_inplace, force_clone=True
+    )
     fgraph.attach_feature(equivalence_tracker)
     return fgraph, updates, equivalence_tracker
 
@@ -1204,7 +1206,9 @@ class _VariableEquivalenceTracker:
         self.fgraph = None
 
     def on_attach(self, fgraph):
-        assert self.fgraph is None
+        if self.fgraph is not None:
+            raise AlreadyThere()
+
         self.equiv = {}
         self.active_nodes = set()
         self.inactive_nodes = set()
@@ -2006,6 +2010,7 @@ class _Maker(FunctionMaker):  # inheritance buys a few helper functions
         fgraph=None,  # If present the optimized graph. we ignore it.
         output_keys=None,
         name=None,
+        no_fgraph_prep=False,
     ):
         self.mode = mode
         self.profile = profile
@@ -2147,6 +2152,7 @@ class _Maker(FunctionMaker):  # inheritance buys a few helper functions
         fgraph.name = name
         self.indices = indices
         self.inputs = inputs
+        # TODO: Get rid of all this `expanded_inputs` nonsense
         self.expanded_inputs = inputs
         self.outputs = outputs
         self.unpack_single = unpack_single
