@@ -204,8 +204,8 @@ def rebuild_collect_shared(
             err_sug = (
                 "If the difference is related to the broadcast pattern,"
                 " you can call the"
-                " tensor.unbroadcast(var, axis_to_unbroadcast[, ...])"
-                " function to remove broadcastable dimensions."
+                " tensor.shape.unbroadcast(var, axis_to_unbroadcast[, ...])"
+                " function to mask broadcastable dimensions."
             )
 
             raise TypeError(err_msg, err_sug)
