@@ -9,7 +9,7 @@ import logging
 import time
 import warnings
 from itertools import chain
-from typing import List, Tuple, Type
+from typing import TYPE_CHECKING, List, Optional, Tuple, Type
 
 import numpy as np
 
@@ -26,7 +26,7 @@ from aesara.graph.basic import (
     graph_inputs,
 )
 from aesara.graph.destroyhandler import DestroyHandler
-from aesara.graph.features import Feature, PreserveVariableAttributes
+from aesara.graph.features import AlreadyThere, Feature, PreserveVariableAttributes
 from aesara.graph.fg import FunctionGraph
 from aesara.graph.op import HasInnerGraph
 from aesara.graph.utils import InconsistencyError, get_variable_trace_string
@@ -34,6 +34,10 @@ from aesara.link.basic import Container
 from aesara.link.utils import raise_with_op
 
 
+if TYPE_CHECKING:
+    from aesara.link.vm import VM
+
+
 _logger = logging.getLogger("aesara.compile.function.types")
 
 
@@ -110,22 +114,27 @@ def fgraph_updated_vars(fgraph, expanded_inputs):
     Reconstruct the full "updates" dictionary, mapping from FunctionGraph input
     variables to the fgraph outputs that will replace their values.
 
+    TODO: Get rid of all this `expanded_inputs` nonsense and use
+    only `fgraph.update_mapping`.
+
     Returns
     -------
     dict variable -> variable
 
     """
     updated_vars = {}
-    potential_values = list(fgraph.outputs)  # copy the list
+
     if len(expanded_inputs) != len(fgraph.inputs):
         raise ValueError("expanded_inputs must match len(fgraph.inputs)")
-    for e_input, ivar in reversed(list(zip(expanded_inputs, fgraph.inputs))):
-        if e_input.update is not None:
-            updated_vars[ivar] = potential_values.pop()
+
+    for out_idx, in_idx in fgraph.update_mapping.items():
+        assert expanded_inputs[in_idx].update is not None
+        updated_vars[fgraph.inputs[in_idx]] = fgraph.outputs[out_idx]
+
     return updated_vars
 
 
-class Supervisor:
+class Supervisor(Feature):
     """
     Listener for FunctionGraph events which makes sure that no
     operation overwrites the contents of protected Variables. The
@@ -134,8 +143,22 @@ class Supervisor:
     """
 
     def __init__(self, protected):
+        self.fgraph = None
         self.protected = list(protected)
 
+    def clone(self):
+        return type(self)(self.protected)
+
+    def on_attach(self, fgraph):
+        if hasattr(fgraph, "_supervisor"):
+            raise AlreadyThere(f"A Supervisor is already attached to {fgraph}.")
+
+        if self.fgraph is not None and self.fgraph != fgraph:
+            raise Exception("This Feature is already associated with a FunctionGraph")
+
+        fgraph._supervisor = self
+        self.fgraph = fgraph
+
     def validate(self, fgraph):
         if config.cycle_detection == "fast" and hasattr(fgraph, "has_destroyers"):
             if fgraph.has_destroyers(self.protected):
@@ -152,7 +175,9 @@ def std_fgraph(
     input_specs: List[SymbolicInput],
     output_specs: List[SymbolicOutput],
     accept_inplace: bool = False,
+    fgraph: Optional[FunctionGraph] = None,
     features: List[Type[Feature]] = [PreserveVariableAttributes],
+    force_clone=False,
 ) -> Tuple[FunctionGraph, List[SymbolicOutput]]:
     """Make or set up `FunctionGraph` corresponding to the input specs and the output specs.
 
@@ -166,26 +191,50 @@ def std_fgraph(
     `accept_inplace` is ``True``, a `DestroyHandler` will be added to the
     `FunctionGraph` if there are any in-place operations.
 
-    The returned FunctionGraph is a clone of the graph between the provided
-    inputs and outputs.
+    If `fgraph` is ``None``, the returned `FunctionGraph` is a clone of the
+    graph between the provided inputs and outputs.
 
     """
-    orig_inputs = [spec.variable for spec in input_specs]
-
     # Extract the updates and the mapping between update outputs and the
     # updated inputs
     updates = []
     update_mapping = {}
     out_idx = len(output_specs)
-    for inp_idx in range(len(input_specs)):
-        if input_specs[inp_idx].update:
-            updates.append(input_specs[inp_idx].update)
-            update_mapping[out_idx] = inp_idx
+    for idx, input_spec in enumerate(input_specs):
+        if input_spec.update:
+            updates.append(input_spec.update)
+            update_mapping[out_idx] = idx
             out_idx += 1
 
-    orig_outputs = [spec.variable for spec in output_specs] + updates
+    found_updates = []
+    if fgraph and fgraph.update_mapping is None:
+        fgraph.update_mapping = update_mapping
+        for update in updates:
+            fgraph.add_output(update, reason="std_fgraph")
 
-    fgraph = FunctionGraph(orig_inputs, orig_outputs, update_mapping=update_mapping)
+        found_updates.extend(map(SymbolicOutput, updates))
+    elif fgraph is None:
+        input_vars = []
+
+        # If one of the inputs is non-atomic (i.e. has a non-`None` `Variable.owner`),
+        # then we need to create/clone the graph starting at these inputs.
+        # The result will be atomic versions of the given inputs connected to
+        # the same outputs.
+        # Otherwise, when all the inputs are already atomic, there's no need to
+        # clone the graph.
+        clone = force_clone
+        for spec in input_specs:
+            input_vars.append(spec.variable)
+            clone |= spec.variable.owner is not None
+
+        fgraph = FunctionGraph(
+            input_vars,
+            [spec.variable for spec in output_specs] + updates,
+            update_mapping=update_mapping,
+            clone=clone,
+        )
+
+        found_updates.extend(map(SymbolicOutput, updates))
 
     for node in fgraph.apply_nodes:
         if node.op.destroy_map:
@@ -210,7 +259,8 @@ def std_fgraph(
     # If named nodes are replaced, keep the name
     for feature in features:
         fgraph.attach_feature(feature())
-    return fgraph, list(map(SymbolicOutput, updates))
+
+    return fgraph, found_updates
 
 
 class AliasedMemoryError(Exception):
@@ -225,42 +275,45 @@ DUPLICATE = object()
 
 
 class Function:
-    """
-    Type of the functions returned by aesara.function or
-    aesara.FunctionMaker.create.
+    r"""A class that wraps the execution of a `VM` making it easier for use as a "function".
 
     `Function` is the callable object that does computation.  It has the storage
     of inputs and outputs, performs the packing and unpacking of inputs and
     return values. It implements the square-bracket indexing so that you can
     look up the value of a symbolic node.
 
-    Functions are copyable via {{{fn.copy()}}} and {{{copy.copy(fn)}}}.
+    Functions are copyable via `Function.copy` and the `copy.copy` interface.
     When a function is copied, this instance is duplicated. Contrast with
     self.maker (instance of `FunctionMaker`) that is shared between copies.
     The meaning of copying a function is that the containers and their current
     values will all be duplicated. This requires that mutable inputs be
     copied, whereas immutable inputs may be shared between copies.
 
-    A Function instance is hashable, on the basis of its memory
-    address (its id).
-
+    A Function instance is hashable, on the basis of its memory address (its
+    id).
     A Function instance is only equal to itself.
-
     A Function instance may be serialized using the `pickle` or
     `cPickle` modules.  This will save all default inputs, the graph,
     and WRITEME to the pickle file.
 
-    A Function instance have a ``trust_input`` field that default to
-    False. When True, we don't do extra check of the input to give
-    better error message. In some case, python code will still return
-    the good results if you pass a python or numpy scalar instead of a
-    numpy tensor.  C code should raise an error if you pass an object
-    of the wrong type.
+    A `Function` instance has a `Function.trust_input` field that defaults to
+    ``False``. When ``True``, the `Function` will skip all checks on the
+    inputs.
 
     Attributes
     ----------
     finder
+        Dictionary mapping several kinds of things to containers.
+
+        We set an entry in finder for:
+        - the index of the input
+        - the variable instance the input is based on
+        - the name of the input
+
+        All entries map to the container or to DUPLICATE if an ambiguity
+        is detected.
     inv_finder
+        Reverse lookup of `finder`.  It maps containers to `SymbolicInput`\s.
 
     """
 
@@ -275,111 +328,59 @@ class Function:
 
     If the value is 'raise', then an AliasedMemoryError will be raised
     if aliased storage is detected during pickle.dump.
-
-    """
-
-    input_storage = None
-    """
-    List of Container instances.
-
-    """
-
-    output_storage = None
-    """
-    List of Container instances.
-
-    """
-
-    indices = None
-    """
-    List of (SymbolicInput, indices, [SymbolicInput,...]),
-    one tuple for each input.
-
-    The first tuple element is the SymbolicInput object for the corresponding
-    function input.
-
-    The second and third tuple elements are used only by Kits, which
-    are deprecated.
-
-    """
-
-    defaults = None
-    """
-    List of 3-tuples, one 3-tuple for each input.
-
-    Tuple element 0: Bool:  Is this input required at each function call?
-    Tuple element 1: Bool: Should this inputs value be reverted after
-        each call?
-    Tuple element 2: Any:  The value associated with this input.
-
-    """
-
-    unpack_single = None
-    """
-    Bool: for outputs lists of length 1, should the 0'th element be
-    returned directly?
-
-    """
-
-    return_none = None
-    """
-    Bool: whether the function should return None or not.
-
-    """
-
-    maker = None
-    """
-    FunctionMaker instance.
-
-    """
-
-    fn = None
-    """
-    A function that evaluates the graph. Typically a linker's make_thunk method
-    created this function.
-
-    """
-
-    finder = None
-    """
-    Dictionary mapping several kinds of things to containers.
-
-    We set an entry in finder for:
-
-    - the index of the input
-
-    - the variable instance the input is based on
-
-    - the name of the input
-
-    All entries map to the container or to DUPLICATE if an ambiguity
-    is detected.
-
-    """
-
-    inv_finder = None
-    """
-    Dict. Reverse lookup of `finder`.
-
-    It maps container -> SymbolicInput
-
     """
 
     def __init__(
         self,
-        fn,
+        vm: "VM",
         input_storage,
         output_storage,
         indices,
         outputs,
         defaults,
-        unpack_single,
-        return_none,
+        unpack_single: bool,
+        return_none: bool,
         output_keys,
-        maker,
-        name=None,
+        maker: "FunctionMaker",
+        name: Optional[str] = None,
     ):
-        self.fn = fn
+        """
+        Parameters
+        ----------
+        vm
+            A `VM` instance that evaluates the graph when called.
+        input_storage
+            List of storage cells for each input.
+        output_storage
+            List of storage cells for each output.
+        indices
+            List of ``(SymbolicInput, indices, [SymbolicInput,...])``, one
+            tuple for each input.  The first tuple element is the `SymbolicInput`
+            object for the corresponding function input.  The second and third
+            tuple elements are used only by Kits, which are deprecated.
+        outputs
+            TODO
+        defaults
+            List of 3-tuples, one 3-tuple for each input.
+            Tuple element 0: ``bool``.  Is this input required at each function
+            call?
+            Tuple element 1: ``bool``.  Should this inputs value be reverted
+            after each call?
+            Tuple element 2: ``Any``.  The value associated with this input.
+        unpack_single
+            For outputs lists of length 1, should the 0'th element be
+            returned directly?
+        return_none
+            Whether the function should return ``None`` or not.
+        output_keys
+            TODO
+        maker
+            The `FunctionMaker` that created this instance.
+        name
+            A string name.
+        """
+        # TODO: Rename to `vm`
+        self.vm = vm
         self.input_storage = input_storage
         self.output_storage = output_storage
         self.indices = indices
@@ -395,7 +396,7 @@ class Function:
         self.output_keys = output_keys
 
         # See if we have any mutable / borrow inputs
-        # TODO: this only need to be set if there is more then 1 input
+        # TODO: this only need to be set if there is more than one input
         self._check_for_aliased_inputs = False
         for i in maker.inputs:
             # If the input is a shared variable, the memory region is
@@ -526,9 +527,10 @@ class Function:
         self._value = ValueAttribute()
         self._container = ContainerAttribute()
 
+        # TODO: Get rid of all this `expanded_inputs` nonsense
         assert len(self.maker.expanded_inputs) == len(self.input_storage)
 
-        # This is used only when `fn.need_update_inputs` is `False`, because
+        # This is used only when `vm.need_update_inputs` is `False`, because
         # we're using one of the VM objects and it is putting updates back into
         # the input containers all by itself.
         self.n_returned_outputs = len(self.output_storage) - sum(
@@ -646,10 +648,15 @@ class Function:
             [memo[o] for o in out_vars],
             clone=False,
         )
+        fg_cpy.update_mapping = maker.fgraph.update_mapping
 
         # Re initialize Outs and swap update and variable in Ins
         # By doing this, we can pass FunctionMaker.check_unused_inputs()
-        outs = list(map(SymbolicOutput, fg_cpy.outputs[: len(maker.outputs)]))
+        if delete_updates:
+            outs = list(map(SymbolicOutput, fg_cpy.outputs[: len(maker.outputs)]))
+        else:
+            outs = list(map(SymbolicOutput, fg_cpy.outputs))
+
         for out_ori, out_cpy in zip(maker.outputs, outs):
             out_cpy.borrow = out_ori.borrow
 
@@ -686,18 +693,21 @@ class Function:
                     fg_cpy.replace(in_v, swap_sv, reason="Swap SV")
 
         # Delete update if needed
-        update_i = len(outs)
-        for i, in_var in zip(ins, fg_cpy.inputs):
-            i.variable = in_var
-            if not delete_updates and i.update is not None:
-                i.update = fg_cpy.outputs[update_i]
-                update_i += 1
+        rev_update_mapping = {v: k for k, v in fg_cpy.update_mapping.items()}
+        for n, (inp, in_var) in enumerate(zip(ins, fg_cpy.inputs)):
+            inp.variable = in_var
+            if not delete_updates and inp.update is not None:
+                out_idx = rev_update_mapping[n]
+                inp.update = fg_cpy.outputs[out_idx]
             else:
-                i.update = None
+                inp.update = None
+
+        if delete_updates:
+            fg_cpy.update_mapping = {}
 
         # Construct new storage_map that map new variable to old storage,
         # so that the ensuing function shares storage with the original one
-        storage_map = self.fn.storage_map
+        storage_map = self.vm.storage_map
         new_storage_map = {}
         # TODO: We could share the output storage, but we must make sure
         # 2 different function call won't override each other values. This
@@ -747,6 +757,7 @@ class Function:
             # can contain inplace. DebugMode check
             # that.
             accept_inplace=True,
+            no_fgraph_prep=True,
         ).create(input_storage, storage_map=new_storage_map)
 
         for in_ori, in_cpy, ori, cpy in zip(
@@ -774,6 +785,8 @@ class Function:
                 f_cpy.finder[swap[in_ori.variable]] = container
                 in_cpy.variable = swap[in_ori.variable]
 
+        f_cpy.trust_input = self.trust_input
+        f_cpy.unpack_single = self.unpack_single
         f_cpy.name = name
         f_cpy.maker.fgraph.name = name
         return f_cpy
@@ -959,24 +972,24 @@ class Function:
         t0_fn = time.time()
         try:
             outputs = (
-                self.fn()
+                self.vm()
                 if output_subset is None
-                else self.fn(output_subset=output_subset)
+                else self.vm(output_subset=output_subset)
             )
         except Exception:
             restore_defaults()
-            if hasattr(self.fn, "position_of_error"):
+            if hasattr(self.vm, "position_of_error"):
                 # this is a new vm-provided function or c linker
                 # they need this because the exception manipulation
                 # done by raise_with_op is not implemented in C.
                 thunk = None
-                if hasattr(self.fn, "thunks"):
-                    thunk = self.fn.thunks[self.fn.position_of_error]
+                if hasattr(self.vm, "thunks"):
+                    thunk = self.vm.thunks[self.vm.position_of_error]
                 raise_with_op(
                     self.maker.fgraph,
-                    node=self.fn.nodes[self.fn.position_of_error],
+                    node=self.vm.nodes[self.vm.position_of_error],
                     thunk=thunk,
-                    storage_map=getattr(self.fn, "storage_map", None),
+                    storage_map=getattr(self.vm, "storage_map", None),
                 )
             else:
                 # old-style linkers raise their own exceptions
@@ -1000,7 +1013,7 @@ class Function:
 
         # if we are allowing garbage collection, remove the
         # output reference from the internal storage cells
-        if getattr(self.fn, "allow_gc", False):
+        if getattr(self.vm, "allow_gc", False):
             assert len(self.output_storage) == len(self.maker.fgraph.outputs)
             for o_container, o_variable in zip(
                 self.output_storage, self.maker.fgraph.outputs
@@ -1010,7 +1023,9 @@ class Function:
                     # WARNING: This circumvents the 'readonly' attribute in x
                     o_container.storage[0] = None
 
-        if getattr(self.fn, "need_update_inputs", True):
+        # TODO: Get rid of this and `expanded_inputs`, since all the VMs now
+        # perform the updates themselves
+        if getattr(self.vm, "need_update_inputs", True):
             # Update the inputs that have an update function
             for input, storage in reversed(
                 list(zip(self.maker.expanded_inputs, self.input_storage))
@@ -1034,8 +1049,8 @@ class Function:
         if profile:
             profile.fct_callcount += 1
             profile.fct_call_time += dt_call
-            if hasattr(self.fn, "update_profile"):
-                self.fn.update_profile(profile)
+            if hasattr(self.vm, "update_profile"):
+                self.vm.update_profile(profile)
             if profile.ignore_first_call:
                 profile.reset()
                 profile.ignore_first_call = False
@@ -1079,10 +1094,10 @@ class Function:
         """
         # 1.no allow_gc return False
         # 2.has allow_gc, if allow_gc is False, return True
-        if not getattr(self.fn, "allow_gc", True):
-            for key in self.fn.storage_map:
+        if not getattr(self.vm, "allow_gc", True):
+            for key in self.vm.storage_map:
                 if not isinstance(key, Constant):
-                    self.fn.storage_map[key][0] = None
+                    self.vm.storage_map[key][0] = None
 
             for node in self.nodes_with_inner_function:
                 if hasattr(node.fn, "free"):
@@ -1378,6 +1393,63 @@ class FunctionMaker:
                         "Valid values are 'raise', 'warn', and 'ignore'."
                     )
 
+    @staticmethod
+    def prepare_fgraph(
+        inputs, outputs, additional_outputs, fgraph, optimizer, linker, profile
+    ):
+
+        try:
+            start_optimizer = time.time()
+
+            optimizer_profile = None
+            opt_time = None
+
+            with config.change_flags(
+                compute_test_value=config.compute_test_value_opt,
+                traceback__limit=config.traceback__compile_limit,
+            ):
+                optimizer_profile = optimizer(fgraph)
+
+                end_optimizer = time.time()
+                opt_time = end_optimizer - start_optimizer
+                _logger.debug(f"Optimizing took {opt_time:f} seconds")
+
+                # Add deep copy to respect the memory interface
+                insert_deepcopy(fgraph, inputs, outputs + additional_outputs)
+        finally:
+
+            # If the optimizer got interrupted
+            if opt_time is None:
+                end_optimizer = time.time()
+                opt_time = end_optimizer - start_optimizer
+
+            aesara.compile.profiling.total_graph_opt_time += opt_time
+
+            if profile:
+                if optimizer_profile is None and hasattr(optimizer, "pre_profile"):
+                    optimizer_profile = optimizer.pre_profile
+
+                profile.optimizer_time += opt_time
+
+                if config.profile_optimizer:
+                    profile.optimizer_profile = (optimizer, optimizer_profile)
+            elif config.profile_optimizer and profile is not False:
+                # If False, it means the profiling for that function was
+                # explicitly disabled
+                warnings.warn(
+                    (
+                        "config.profile_optimizer requires config.profile to "
+                        " be set to True as well"
+                    ),
+                    stacklevel=3,
+                )
+
+        if not hasattr(linker, "accept"):
+            raise ValueError(
+                "'linker' parameter of FunctionMaker should be "
+                f"a Linker with an accept method or one of {list(aesara.compile.mode.predefined_linkers.keys())}"
+            )
+
     def __init__(
         self,
         inputs,
@@ -1390,6 +1462,7 @@ class FunctionMaker:
         fgraph=None,
         output_keys=None,
         name=None,
+        no_fgraph_prep=False,
     ):
         # Save the provided mode, not the instantiated mode.
         # The instantiated mode don't pickle and if we unpickle an Aesara
@@ -1433,84 +1506,32 @@ class FunctionMaker:
 
         indices = [[input, None, [input]] for input in inputs]
 
-        if fgraph is None:
-            need_opt = True
-            # make the fgraph (copies the graph, creates NEW INPUT AND
-            # OUTPUT VARIABLES)
-            fgraph, additional_outputs = std_fgraph(inputs, outputs, accept_inplace)
+        fgraph, found_updates = std_fgraph(
+            inputs, outputs, accept_inplace, fgraph=fgraph
+        )
+
+        if fgraph.profile is None:
             fgraph.profile = profile
-        else:
-            # fgraph is already an optimized one
-            need_opt = False
-            updates = [spec.update for spec in inputs if spec.update]
-            additional_outputs = list(map(SymbolicOutput, updates))
 
         self.fgraph = fgraph
 
         optimizer, linker = mode.optimizer, copy.copy(mode.linker)
-        if need_opt:
-            # Why we add stack on node when it get done in output var?
-            try:
-                start_optimizer = time.time()
 
-                # In case there is an error during optimization.
-                optimizer_profile = None
-                opt_time = None
-
-                with config.change_flags(
-                    compute_test_value=config.compute_test_value_opt,
-                    traceback__limit=config.traceback__compile_limit,
-                ):
-                    optimizer_profile = optimizer(fgraph)
-
-                    end_optimizer = time.time()
-                    opt_time = end_optimizer - start_optimizer
-                    _logger.debug(f"Optimizing took {opt_time:f} seconds")
-
-                    # Add deep copy to respect the memory interface
-                    insert_deepcopy(fgraph, inputs, outputs + additional_outputs)
-            finally:
-
-                # If the optimizer got interrupted
-                if opt_time is None:
-                    end_optimizer = time.time()
-                    opt_time = end_optimizer - start_optimizer
-
-                aesara.compile.profiling.total_graph_opt_time += opt_time
-
-                if profile:
-                    if optimizer_profile is None and hasattr(optimizer, "pre_profile"):
-                        optimizer_profile = optimizer.pre_profile
-
-                    profile.optimizer_time += opt_time
-
-                    if config.profile_optimizer:
-                        profile.optimizer_profile = (optimizer, optimizer_profile)
-                # IF False, if mean the profile for that function was
-                # explicitly disabled
-                elif config.profile_optimizer and profile is not False:
-                    warnings.warn(
-                        (
-                            "config.profile_optimizer requires config.profile to "
-                            " be set to True as well"
-                        ),
-                        stacklevel=3,
-                    )
-
-        if not hasattr(linker, "accept"):
-            raise ValueError(
-                "'linker' parameter of FunctionMaker should be "
-                f"a Linker with an accept method or one of {list(aesara.compile.mode.predefined_linkers.keys())}"
+        if not no_fgraph_prep:
+            self.prepare_fgraph(
+                inputs, outputs, found_updates, fgraph, optimizer, linker, profile
             )
 
-        # the 'no_borrow' outputs are the ones for which that we can't
+        assert len(fgraph.outputs) == len(outputs + found_updates)
+
+        # The 'no_borrow' outputs are the ones for which that we can't
         # return the internal storage pointer.
-        assert len(fgraph.outputs) == len(outputs + additional_outputs)
         no_borrow = [
             output
-            for output, spec in zip(fgraph.outputs, outputs + additional_outputs)
+            for output, spec in zip(fgraph.outputs, outputs + found_updates)
             if not spec.borrow
         ]
+
         if no_borrow:
             self.linker = linker.accept(
                 fgraph,
@@ -1521,11 +1542,15 @@ class FunctionMaker:
             self.linker = linker.accept(fgraph, profile=profile)
 
         if hasattr(linker, "accept_var_updates"):
-            # hacky thing so VMLinker knows about updates
+            # TODO: This is a hack that makes `VMLinker` aware of updates;
+            # clean this up.
             self.linker.accept_var_updates(fgraph_updated_vars(fgraph, inputs))
+
         fgraph.name = name
         self.indices = indices
         self.inputs = inputs
+
+        # TODO: Get rid of all this `expanded_inputs` nonsense
         self.expanded_inputs = inputs
         self.outputs = outputs
         self.unpack_single = unpack_single
@@ -1670,6 +1695,7 @@ def orig_function(
     profile=None,
     on_unused_input=None,
     output_keys=None,
+    fgraph: Optional[FunctionGraph] = None,
 ) -> Function:
     """
     Return a Function that will calculate the outputs from the inputs.
@@ -1731,6 +1757,7 @@ def orig_function(
             on_unused_input=on_unused_input,
             output_keys=output_keys,
             name=name,
+            fgraph=fgraph,
         )
         with config.change_flags(compute_test_value="off"):
             fn = m.create(defaults)
