@@ -2,6 +2,7 @@
 import time
 from collections import OrderedDict
 from typing import (
+    TYPE_CHECKING,
     Any,
     Dict,
     Iterable,
@@ -18,7 +19,7 @@ from typing_extensions import Literal
 
 import aesara
 from aesara.configdefaults import config
-from aesara.graph.basic import Apply, Constant, Node, Variable, applys_between
+from aesara.graph.basic import Apply, AtomicVariable, Variable, applys_between
 from aesara.graph.basic import as_string as graph_as_string
 from aesara.graph.basic import clone_get_equiv, graph_inputs, io_toposort, vars_between
 from aesara.graph.features import AlreadyThere, Feature, ReplaceValidate
@@ -26,6 +27,9 @@ from aesara.graph.utils import MetaObject, MissingInputError, TestValueError
 from aesara.misc.ordered_set import OrderedSet
 
 
+if TYPE_CHECKING:
+    from aesara.graph.op import Op
+
 ApplyOrOutput = Union[Apply, Literal["output"]]
 ClientType = Tuple[ApplyOrOutput, int]
 
@@ -69,9 +73,7 @@ class FunctionGraph(MetaObject):
         features: Optional[Sequence[Feature]] = None,
         clone: bool = True,
         update_mapping: Optional[Dict[Variable, Variable]] = None,
-        memo: Optional[Dict[Variable, Variable]] = None,
-        copy_inputs: bool = True,
-        copy_orphans: bool = True,
+        **clone_kwds,
     ):
         """
         Create a `FunctionGraph` which operates on the subgraph between the
@@ -83,33 +85,29 @@ class FunctionGraph(MetaObject):
             Input variables of the graph.
         outputs
             Output variables of the graph.
-        clone
-            If ``True``, the graph will be cloned.
         features
             A list of features to be added to the `FunctionGraph`.
+        clone
+            If ``True``, the graph will be cloned.
         update_mapping
             Mapping between the `inputs` with updates and the `outputs`
             corresponding to their updates.
-        memo
-            See :func:`aesara.graph.basic.clone_get_equiv`.
-        copy_inputs
-            See :func:`aesara.graph.basic.clone_get_equiv`.
-        copy_orphans
-            See :func:`aesara.graph.basic.clone_get_equiv`.
+        clone_kwds
+            Keywords passed to `clone_get_equiv` when `clone` is ``True``.
         """
         if outputs is None:
             raise ValueError("No outputs specified")
 
         if inputs is None:
-            inputs = [i for i in graph_inputs(outputs) if not isinstance(i, Constant)]
+            inputs = [
+                i for i in graph_inputs(outputs) if not isinstance(i, AtomicVariable)
+            ]
 
         if clone:
             _memo = clone_get_equiv(
                 inputs,
                 outputs,
-                copy_inputs=copy_inputs,
-                copy_orphans=copy_orphans,
-                memo=cast(Dict[Node, Node], memo),
+                **clone_kwds,
             )
             outputs = [cast(Variable, _memo[o]) for o in outputs]
             inputs = [cast(Variable, _memo[i]) for i in inputs]
@@ -126,13 +124,13 @@ class FunctionGraph(MetaObject):
         # outputs are cached in this field
         self.apply_nodes: Set[Apply] = set()
 
-        # Ditto for variable nodes.
-        # It must contain all fgraph.inputs and all apply_nodes
-        # outputs even if they aren't used in the graph.
+        # It includes inputs, outputs, and all intermediate variables
+        # connecting the inputs and outputs.  It also contains irrelevant
+        # outputs the nodes in `self.apply_nodes`.
         self.variables: Set[Variable] = set()
 
         self.inputs: List[Variable] = []
-        self.outputs: List[Variable] = list(outputs)
+        self.outputs: List[Variable] = []
         self.clients: Dict[Variable, List[ClientType]] = {}
 
         for f in features:
@@ -152,13 +150,19 @@ class FunctionGraph(MetaObject):
             self.add_input(in_var, check=False)
 
         for output in outputs:
-            self.import_var(output, reason="init")
-        for i, output in enumerate(outputs):
-            self.clients[output].append(("output", i))
+            self.add_output(output, reason="init")
 
         self.profile = None
         self.update_mapping = update_mapping
 
+    def add_output(
+        self, var: Variable, reason: Optional[str] = None, import_missing: bool = False
+    ):
+        """Add a new variable as an output to this `FunctionGraph`."""
+        self.outputs.append(var)
+        self.import_var(var, reason=reason, import_missing=import_missing)
+        self.clients[var].append(("output", len(self.outputs) - 1))
+
     def add_input(self, var: Variable, check: bool = True) -> None:
         """Add a new variable as an input to this `FunctionGraph`.
 
@@ -172,7 +176,6 @@ class FunctionGraph(MetaObject):
 
         self.inputs.append(var)
         self.setup_var(var)
-        self.variables.add(var)
 
     def setup_var(self, var: Variable) -> None:
         """Set up a variable so it belongs to this `FunctionGraph`.
@@ -210,6 +213,7 @@ class FunctionGraph(MetaObject):
         var: Variable,
         client_to_remove: ClientType,
         reason: Optional[str] = None,
+        remove_if_empty: bool = False,
     ) -> None:
         """Recursively remove clients of a variable.
 
@@ -222,11 +226,14 @@ class FunctionGraph(MetaObject):
 
         Parameters
         ----------
-        var : Variable
+        var
             The clients of `var` that will be removed.
-        client_to_remove : pair of (Apply, int)
+        client_to_remove
             A ``(node, i)`` pair such that ``node.inputs[i]`` will no longer be
             `var` in this `FunctionGraph`.
+        remove_if_empty
+            When ``True``, if `var`'s `Apply` node is removed, remove the
+            entry for `var` in `self.clients`.
 
         """
 
@@ -250,8 +257,6 @@ class FunctionGraph(MetaObject):
             # Now, `var` has no more clients, so check if we need to remove it
             # and its `Apply` node
             if not var.owner:
-                # The `var` is a `Constant` or an input without a client, so we
-                # remove it
                 self.variables.remove(var)
             else:
                 apply_node = var.owner
@@ -274,12 +279,15 @@ class FunctionGraph(MetaObject):
                     for i, in_var in enumerate(apply_node.inputs):
                         removal_stack.append((in_var, (apply_node, i)))
 
+                    if remove_if_empty:
+                        del self.clients[var]
+
     def import_var(
         self, var: Variable, reason: Optional[str] = None, import_missing: bool = False
     ) -> None:
-        """Import variables into this `FunctionGraph`.
+        """Import a `Variable` into this `FunctionGraph`.
 
-        This will also import the `variable`'s `Apply` node.
+        This will import the `var`'s `Apply` node and inputs.
 
         Parameters
         ----------
@@ -296,7 +304,7 @@ class FunctionGraph(MetaObject):
             self.import_node(var.owner, reason=reason, import_missing=import_missing)
         elif (
             var.owner is None
-            and not isinstance(var, Constant)
+            and not isinstance(var, AtomicVariable)
             and var not in self.inputs
         ):
             from aesara.graph.null_type import NullType
@@ -344,7 +352,7 @@ class FunctionGraph(MetaObject):
                 for var in node.inputs:
                     if (
                         var.owner is None
-                        and not isinstance(var, Constant)
+                        and not isinstance(var, AtomicVariable)
                         and var not in self.inputs
                     ):
                         if import_missing:
@@ -505,9 +513,6 @@ class FunctionGraph(MetaObject):
                     )
 
         for node, i in list(self.clients[var]):
-            assert (node == "output" and self.outputs[i] is var) or (
-                isinstance(node, Apply) and node.inputs[i] is var
-            )
             self.change_node_input(
                 node, i, new_var, reason=reason, import_missing=import_missing
             )
@@ -517,6 +522,147 @@ class FunctionGraph(MetaObject):
         for var, new_var in pairs:
             self.replace(var, new_var, **kwargs)
 
+    def _remove_output(self, idx: int):
+        """Remove the output at index `idx` and update the indices in the clients entries.
+
+        `FunctionGraph.clients` contains entries like ``("output", i)`` under
+        each output variable in `FunctionGraph.outputs`.  The ``i`` values
+        correspond to each output's location within the `FunctionGraph.outputs`
+        list, so, when an output is removed from the graph, all these entries
+        need to be updated.  This method performs those updates.
+
+        TODO: We could track these entries in a new instance attribute and make
+        them lists, then each could be updated in-place very easily.  This
+        seems fine, because the `FunctionGraph.clients` ``dict`` and list in
+        which they're contained are already being updated in-place.
+        """
+        old_idx_mappings = tuple((out, i) for i, out in enumerate(self.outputs))
+        self.outputs.pop(idx)
+
+        new_idx = 0
+        for (out, old_idx) in old_idx_mappings:
+            if old_idx == idx:
+                continue
+            out_clients = self.clients[out]
+            arrow: ClientType = ("output", old_idx)
+            arrow_idx = out_clients.index(arrow)
+            out_clients[arrow_idx] = ("output", new_idx)
+            new_idx += 1
+
+    def remove_node(self, node: Apply, reason: Optional[str] = None):
+        """Remove an `Apply` node from the `FunctionGraph`.
+
+        This will remove everything that depends on the outputs of `node`, as
+        well as any "orphaned" variables and nodes created by `node`'s removal.
+        """
+
+        if node not in self.apply_nodes:
+            return
+
+        self.apply_nodes.remove(node)
+
+        if not hasattr(node.tag, "removed_by"):
+            node.tag.removed_by = []
+
+        node.tag.removed_by.append(str(reason))
+
+        # Remove the outputs of the node (i.e. everything "below" it)
+        for out in node.outputs:
+            self.variables.remove(out)
+
+            out_clients = self.clients.get(out, ())
+            while out_clients:
+                out_client, out_idx = out_clients.pop()
+
+                if out_client == "output":
+
+                    self._remove_output(out_idx)
+
+                    # TODO: We could short-circuit all of the graph walking and
+                    # clear everything at once when all the outputs are gone.
+                    # if not self.outputs:
+                    #     self.clients = {inp: [] for inp in self.inputs}
+                    #     self.variables = set()
+                    #     while self.apply_nodes:
+                    #         node = self.apply_nodes.pop()
+                    #         if not hasattr(node.tag, "removed_by"):
+                    #             node.tag.removed_by = []
+                    #
+                    #         node.tag.removed_by.append(str(reason))
+                    #
+                    #         self.execute_callbacks("on_prune", node, reason)
+                else:
+                    assert isinstance(out_client, Apply)
+                    self.remove_node(out_client, reason=reason)
+
+            if out in self.clients:
+                del self.clients[out]
+
+        # Remove all the arrows pointing to this `node`, and any orphaned
+        # variables created by removing those arrows
+        for inp_idx, inp in enumerate(node.inputs):
+
+            inp_clients: List[ClientType] = self.clients.get(inp, [])
+
+            arrow = (node, inp_idx)
+
+            if arrow not in inp_clients:
+                continue
+
+            inp_clients.remove(arrow)
+
+            if not inp_clients and inp not in self.outputs:
+                if inp.owner:
+                    # If this input has no clients (after removing this arrow),
+                    # is not an input (i.e. it has a non-`None` owner) or an
+                    # output to the `FunctionGraph`, then it's an orphan
+
+                    # We need to check whether or not this orphaned input's
+                    # node is still needed in the graph
+                    inp_node = inp.owner
+
+                    if not any(
+                        out in self.variables
+                        for out in inp_node.outputs
+                        if out is not inp
+                    ):
+                        self.remove_node(inp_node, reason=reason)
+                else:
+                    # This is an unused input
+                    self.variables.remove(inp)
+
+        # The callbacks be triggered after everything has been removed so that
+        # the `FunctionGraph` state subscribers see is valid.
+        self.execute_callbacks("on_prune", node, reason)
+
+    def remove_input(self, input_idx: int, reason: Optional[str] = None):
+        """Remove the input at index `input_idx`."""
+        var = self.inputs.pop(input_idx)
+
+        for client, idx in list(self.clients[var]):
+            if client == "output":
+                out_var = self.outputs[idx]
+                out_node = out_var.owner
+                if out_node is None:
+                    assert out_var in self.inputs
+                    self.outputs.pop(idx)
+                    continue
+                client_node = out_node
+            else:
+                assert isinstance(client, Apply)
+                client_node = client
+
+            self.remove_node(client_node, reason=reason)
+
+    def remove_output(self, output_idx: int, reason: Optional[str] = None):
+        """Remove the output at index `input_idx`."""
+
+        var = self.outputs[output_idx]
+        self._remove_output(output_idx)
+        self.remove_client(
+            var, ("output", output_idx), reason=reason, remove_if_empty=True
+        )
+
     def attach_feature(self, feature: Feature) -> None:
         """Add a ``graph.features.Feature`` to this function graph and trigger its ``on_attach`` callback."""
         # Filter out literally identical `Feature`s
@@ -668,9 +814,7 @@ class FunctionGraph(MetaObject):
             nodes_missing = nodes.difference(self.apply_nodes)
             nodes_excess = self.apply_nodes.difference(nodes)
             raise Exception(
-                "The nodes are inappropriately cached. missing, in excess: ",
-                nodes_missing,
-                nodes_excess,
+                f"The following nodes are inappropriately cached:\nmissing: {nodes_missing}\nin excess: {nodes_excess}"
             )
         for node in nodes:
             for i, variable in enumerate(node.inputs):
@@ -684,15 +828,13 @@ class FunctionGraph(MetaObject):
             vars_missing = variables.difference(self.variables)
             vars_excess = self.variables.difference(variables)
             raise Exception(
-                "The variables are inappropriately cached. missing, in excess: ",
-                vars_missing,
-                vars_excess,
+                f"The following variables are inappropriately cached:\nmissing: {vars_missing}\nin excess: {vars_excess}"
             )
         for variable in variables:
             if (
                 variable.owner is None
                 and variable not in self.inputs
-                and not isinstance(variable, Constant)
+                and not isinstance(variable, AtomicVariable)
             ):
                 raise Exception(f"Undeclared input: {variable}")
             for cl_node, i in self.clients[variable]:
@@ -722,39 +864,42 @@ class FunctionGraph(MetaObject):
         return self.clone_get_equiv(check_integrity)[0]
 
     def clone_get_equiv(
-        self, check_integrity: bool = True, attach_feature: bool = True
-    ) -> Tuple["FunctionGraph", Dict[Node, Node]]:
+        self, check_integrity: bool = True, attach_feature: bool = True, **kwargs
+    ) -> Tuple[
+        "FunctionGraph",
+        Dict[Union[Apply, Variable, "Op"], Union[Apply, Variable, "Op"]],
+    ]:
         """Clone the graph and return a ``dict`` that maps old nodes to new nodes.
 
         Parameters
         ----------
         check_integrity
-            Whether to check integrity.
+            Whether or not to check the resulting graph's integrity.
         attach_feature
-            Whether to attach feature of origin graph to cloned graph.
+            Whether or not to attach `self`'s features to the cloned graph.
 
         Returns
         -------
         e
-            Cloned fgraph. Every node in cloned graph is cloned.
+            The cloned `FunctionGraph`. Every node in the cloned graph is cloned.
         equiv
             A ``dict`` that maps old nodes to the new nodes.
         """
-        equiv = clone_get_equiv(self.inputs, self.outputs)
+        equiv = clone_get_equiv(self.inputs, self.outputs, **kwargs)
 
-        if check_integrity:
-            self.check_integrity()
         e = FunctionGraph(
             [cast(Variable, equiv[i]) for i in self.inputs],
             [cast(Variable, equiv[o]) for o in self.outputs],
             clone=False,
+            update_mapping=self.update_mapping,
         )
+
         if check_integrity:
             e.check_integrity()
 
         if attach_feature:
             for feature in self._features:
-                e.attach_feature(feature)
+                e.attach_feature(feature.clone())
         return e, equiv
 
     def __getstate__(self):
