@@ -2,11 +2,10 @@ import inspect
 from functools import singledispatch
 from numbers import Number
 from textwrap import indent
-from typing import Union
+from typing import Any, Callable, Optional, Union
 
 import numba
 import numpy as np
-from numba.cpython.unsafe.tuple import tuple_setitem
 
 from aesara import config
 from aesara.graph.basic import Apply
@@ -127,8 +126,14 @@ if {res}[{idx}] > {arr}:
 """
 
 
-def create_vectorize_func(op, node, use_signature=False, identity=None, **kwargs):
-    scalar_op_fn = numba_funcify(op.scalar_op, node=node, inline="always", **kwargs)
+def create_vectorize_func(
+    scalar_op_fn: Callable,
+    node: Apply,
+    use_signature: bool = False,
+    identity: Optional[Any] = None,
+    **kwargs,
+) -> Callable:
+    r"""Create a vectorized Numba function from a `Apply`\s Python function."""
 
     if len(node.outputs) > 1:
         raise NotImplementedError(
@@ -420,7 +425,8 @@ def create_axis_apply_fn(fn, axis, ndim, dtype):
 @numba_funcify.register(Elemwise)
 def numba_funcify_Elemwise(op, node, **kwargs):
 
-    elemwise_fn = create_vectorize_func(op, node, use_signature=False)
+    scalar_op_fn = numba_funcify(op.scalar_op, node=node, inline="always", **kwargs)
+    elemwise_fn = create_vectorize_func(scalar_op_fn, node, use_signature=False)
     elemwise_fn_name = elemwise_fn.__name__
 
     if op.inplace_pattern:
@@ -481,7 +487,15 @@ def numba_funcify_CAReduce(op, node, **kwargs):
 
     np_acc_dtype = np.dtype(acc_dtype)
 
-    scalar_op_identity = np.asarray(op.scalar_op.identity, dtype=np_acc_dtype)
+    scalar_op_identity = op.scalar_op.identity
+    if np_acc_dtype.kind == "i" and not np.isfinite(scalar_op_identity):
+        if np.isposinf(scalar_op_identity):
+            scalar_op_identity = np.iinfo(np_acc_dtype).max
+        else:
+            scalar_op_identity = np.iinfo(np_acc_dtype).min
+
+    # Make sure it has the correct dtype
+    scalar_op_identity = np.array(scalar_op_identity, dtype=np_acc_dtype)
 
     input_name = get_name_for_object(node.inputs[0])
     ndim = node.inputs[0].ndim
@@ -512,10 +526,10 @@ def numba_funcify_DimShuffle(op, **kwargs):
         @numba_basic.numba_njit
         def populate_new_shape(i, j, new_shape, shuffle_shape):
             if i in augment:
-                new_shape = tuple_setitem(new_shape, i, 1)
+                new_shape = numba_basic.tuple_setitem(new_shape, i, 1)
                 return j, new_shape
             else:
-                new_shape = tuple_setitem(new_shape, i, shuffle_shape[j])
+                new_shape = numba_basic.tuple_setitem(new_shape, i, shuffle_shape[j])
                 return j + 1, new_shape
 
     else:
@@ -525,7 +539,7 @@ def numba_funcify_DimShuffle(op, **kwargs):
         # To avoid this compile-time error, we omit the expression altogether.
         @numba_basic.numba_njit(inline="always")
         def populate_new_shape(i, j, new_shape, shuffle_shape):
-            return j, tuple_setitem(new_shape, i, 1)
+            return j, numba_basic.tuple_setitem(new_shape, i, 1)
 
     if ndim_new_shape > 0:
         create_zeros_tuple = numba_basic.create_tuple_creator(
