@@ -96,6 +96,8 @@ def make_numba_random_fn(node, np_random_func):
     The functions generated here add parameter broadcasting and the ``size``
     argument to the Numba-supported scalar ``np.random`` functions.
     """
+    if not isinstance(node.inputs[0], (RandomStateType, RandomStateSharedVariable)):
+        raise TypeError("Numba does not support NumPy `Generator`s")
 
     tuple_size = int(get_vector_length(node.inputs[1]))
     size_dims = tuple_size - max(i.ndim for i in node.inputs[3:])
@@ -204,11 +206,9 @@ def {sized_fn_name}({random_fn_input_names}):
 @numba_funcify.register(aer.PoissonRV)
 @numba_funcify.register(aer.GeometricRV)
 @numba_funcify.register(aer.HyperGeometricRV)
-@numba_funcify.register(aer.CauchyRV)
 @numba_funcify.register(aer.WaldRV)
 @numba_funcify.register(aer.LaplaceRV)
 @numba_funcify.register(aer.BinomialRV)
-@numba_funcify.register(aer.NegBinomialRV)
 @numba_funcify.register(aer.MultinomialRV)
 @numba_funcify.register(aer.RandIntRV)  # only the first two arguments are supported
 @numba_funcify.register(aer.ChoiceRV)  # the `p` argument is not supported
@@ -217,9 +217,6 @@ def numba_funcify_RandomVariable(op, node, **kwargs):
     name = op.name
     np_random_func = getattr(np.random, name)
 
-    if not isinstance(node.inputs[0], (RandomStateType, RandomStateSharedVariable)):
-        raise TypeError("Numba does not support NumPy `Generator`s")
-
     return make_numba_random_fn(node, np_random_func)
 
 
@@ -271,6 +268,19 @@ def {np_random_fn_name}({np_input_names}):
     return make_numba_random_fn(node, np_random_fn)
 
 
+@numba_funcify.register(aer.NegBinomialRV)
+def numba_funcify_NegBinomialRV(op, node, **kwargs):
+    return make_numba_random_fn(node, np.random.negative_binomial)
+
+
+@numba_funcify.register(aer.CauchyRV)
+def numba_funcify_CauchyRV(op, node, **kwargs):
+    def body_fn(loc, scale):
+        return f"    return ({loc} + np.random.standard_cauchy()) / {scale}"
+
+    return create_numba_random_fn(op, node, body_fn)
+
+
 @numba_funcify.register(aer.HalfNormalRV)
 def numba_funcify_HalfNormalRV(op, node, **kwargs):
     def body_fn(a, b):
@@ -297,3 +307,81 @@ def numba_funcify_BernoulliRV(op, node, **kwargs):
         body_fn,
         {"out_dtype": out_dtype, "direct_cast": numba_basic.direct_cast},
     )
+
+
+@numba_funcify.register(aer.CategoricalRV)
+def numba_funcify_CategoricalRV(op, node, **kwargs):
+    out_dtype = node.outputs[1].type.numpy_dtype
+
+    ind_shape_len = node.inputs[3].type.ndim - 1
+    neg_ind_shape_len = -ind_shape_len
+
+    size_len = int(get_vector_length(node.inputs[1]))
+
+    @numba_basic.numba_njit
+    def categorical_rv(rng, size, dtype, p):
+
+        size_tpl = numba_ndarray.to_fixed_tuple(size, size_len)
+        ind_shape = p.shape[:-1]
+
+        if ind_shape_len > 0:
+            if size_len > 0 and size_tpl[neg_ind_shape_len:] != ind_shape:
+                raise ValueError("Parameters shape and size do not match.")
+
+            samples_shape = size_tpl[:neg_ind_shape_len] + ind_shape
+            p_bcast = np.broadcast_to(p, size_tpl[:neg_ind_shape_len] + p.shape)
+        else:
+            samples_shape = size_tpl
+            p_bcast = p
+
+        unif_samples = np.random.uniform(0, 1, samples_shape)
+
+        res = np.empty(samples_shape, dtype=out_dtype)
+        for idx in np.ndindex(*samples_shape):
+            res[idx] = np.searchsorted(np.cumsum(p_bcast[idx]), unif_samples[idx])
+
+        return (rng, res)
+
+    return categorical_rv
+
+
+@numba_funcify.register(aer.DirichletRV)
+def numba_funcify_DirichletRV(op, node, **kwargs):
+
+    out_dtype = node.outputs[1].type.numpy_dtype
+    alphas_ndim = node.inputs[3].type.ndim
+    neg_ind_shape_len = -alphas_ndim + 1
+    size_len = int(get_vector_length(node.inputs[1]))
+
+    if alphas_ndim > 1:
+
+        @numba_basic.numba_njit
+        def dirichlet_rv(rng, size, dtype, alphas):
+
+            if size_len > 0:
+                size_tpl = numba_ndarray.to_fixed_tuple(size, size_len)
+                if (
+                    0 < alphas.ndim - 1 <= len(size_tpl)
+                    and size_tpl[neg_ind_shape_len:] != alphas.shape[:-1]
+                ):
+                    raise ValueError("Parameters shape and size do not match.")
+                samples_shape = size_tpl + alphas.shape[-1:]
+            else:
+                samples_shape = alphas.shape
+
+            res = np.empty(samples_shape, dtype=out_dtype)
+            alphas_bcast = np.broadcast_to(alphas, samples_shape)
+
+            for index in np.ndindex(*samples_shape[:-1]):
+                res[index] = np.random.dirichlet(alphas_bcast[index])
+
+            return (rng, res)
+
+    else:
+
+        @numba_basic.numba_njit
+        def dirichlet_rv(rng, size, dtype, alphas):
+            size = numba_ndarray.to_fixed_tuple(size, size_len)
+            return (rng, np.random.dirichlet(alphas, size))
+
+    return dirichlet_rv
