@@ -34,7 +34,7 @@ def array0d_range(x):
 
 @numba_funcify.register(Scan)
 def numba_funcify_Scan(op, node, **kwargs):
-    inner_fg = FunctionGraph(op.inputs, op.outputs)
+    inner_fg = FunctionGraph(op.inner_inputs, op.inner_outputs)
     numba_at_inner_func = numba_basic.numba_njit(numba_funcify(inner_fg, **kwargs))
 
     n_seqs = op.info.n_seqs
@@ -120,7 +120,7 @@ def numba_funcify_Scan(op, node, **kwargs):
     ]
 
     while_logic = ""
-    if op.as_while:
+    if op.info.as_while:
         # The inner function will be returning a boolean as last argument
         inner_out_indexed.append("while_flag")
         while_logic += """
