@@ -20,10 +20,13 @@ from aesara.scalar.basic import (
     complex_types,
     discrete_types,
     exp,
+    expm1,
     float64,
     float_types,
+    isinf,
     log,
     log1p,
+    switch,
     true_div,
     upcast,
     upgrade_to_float,
@@ -199,13 +202,14 @@ class Erfinv(UnaryScalarOp):
         )
         return (gz * cst * exp(erfinv(x) ** 2),)
 
-    # TODO: erfinv() is not provided by the C standard library
-    # def c_code(self, node, name, inp, out, sub):
-    #    x, = inp
-    #    z, = out
-    #    if node.inputs[0].type in complex_types:
-    #        raise NotImplementedError('type not supported', type)
-    #    return "%(z)s = erfinv(%(x)s);" % locals()
+    def c_code(self, node, name, inp, out, sub):
+        # TODO: erfinv() is not provided by the C standard library
+        # x, = inp
+        # z, = out
+        # if node.inputs[0].type in complex_types:
+        #     raise NotImplementedError('type not supported', type)
+        # return "%(z)s = erfinv(%(x)s);" % locals()
+        raise NotImplementedError()
 
 
 erfinv = Erfinv(upgrade_to_float_no_complex, name="erfinv")
@@ -233,13 +237,14 @@ class Erfcinv(UnaryScalarOp):
         )
         return (-gz * cst * exp(erfcinv(x) ** 2),)
 
-    # TODO: erfcinv() is not provided by the C standard library
-    # def c_code(self, node, name, inp, out, sub):
-    #    x, = inp
-    #    z, = out
-    #    if node.inputs[0].type in complex_types:
-    #        raise NotImplementedError('type not supported', type)
-    #    return "%(z)s = erfcinv(%(x)s);" % locals()
+    def c_code(self, node, name, inp, out, sub):
+        # TODO: erfcinv() is not provided by the C standard library
+        # x, = inp
+        # z, = out
+        # if node.inputs[0].type in complex_types:
+        #     raise NotImplementedError('type not supported', type)
+        # return "%(z)s = erfcinv(%(x)s);" % locals()
+        raise NotImplementedError()
 
 
 erfcinv = Erfcinv(upgrade_to_float_no_complex, name="erfcinv")
@@ -654,7 +659,7 @@ class GammaIncDer(BinaryScalarOp):
         if x == 0:
             return 0
 
-        sqrt_exp = -756 - x ** 2 + 60 * x
+        sqrt_exp = -756 - x**2 + 60 * x
         if (
             (k < 0.8 and x > 15)
             or (k < 12 and x > 30)
@@ -709,6 +714,9 @@ class GammaIncDer(BinaryScalarOp):
         )
         return np.nan
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 gammainc_der = GammaIncDer(upgrade_to_float, name="gammainc_der")
 
@@ -789,6 +797,9 @@ class GammaIncCDer(BinaryScalarOp):
     def impl(self, k, x):
         return self.st_impl(k, x)
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 gammaincc_der = GammaIncCDer(upgrade_to_float, name="gammaincc_der")
 
@@ -897,6 +908,9 @@ class Jv(BinaryScalarOp):
             gz * (jv(v - 1, x) - jv(v + 1, x)) / 2.0,
         ]
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 jv = Jv(upgrade_to_float, name="jv")
 
@@ -985,6 +999,9 @@ class Iv(BinaryScalarOp):
             gz * (iv(v - 1, x) + iv(v + 1, x)) / 2.0,
         ]
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 iv = Iv(upgrade_to_float, name="iv")
 
@@ -1008,6 +1025,9 @@ class I1(UnaryScalarOp):
         (gz,) = grads
         return [gz * (i0(x) + iv(2, x)) / 2.0]
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 i1 = I1(upgrade_to_float, name="i1")
 
@@ -1031,6 +1051,9 @@ class I0(UnaryScalarOp):
         (gz,) = grads
         return [gz * i1(x)]
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 i0 = I0(upgrade_to_float, name="i0")
 
@@ -1201,7 +1224,10 @@ class Log1mexp(UnaryScalarOp):
     def grad(self, inp, grads):
         (x,) = inp
         (gz,) = grads
-        return [gz * true_div(1.0, 1.0 - exp(-x))]
+        res = true_div(-1.0, expm1(-x))
+        # Correct gradient at 0.0 to be -inf
+        res = switch(isinf(res), -np.inf, res)
+        return [gz * res]
 
     def c_code(self, node, name, inp, out, sub):
         (x,) = inp
@@ -1245,6 +1271,9 @@ class BetaInc(ScalarOp):
             ),
         ]
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 betainc = BetaInc(upgrade_to_float_no_complex, name="betainc")
 
@@ -1272,7 +1301,7 @@ class BetaIncDer(ScalarOp):
                 return p * f * (q - 1) / (q * (p + 1))
 
             p2n = p + 2 * n
-            F1 = p ** 2 * f ** 2 * (n - 1) / (q ** 2)
+            F1 = p**2 * f**2 * (n - 1) / (q**2)
             F2 = (
                 (p + q + n - 2)
                 * (p + n - 1)
@@ -1303,18 +1332,18 @@ class BetaIncDer(ScalarOp):
             if n == 1:
                 return -p * f * (q - 1) / (q * (p + 1) ** 2)
 
-            pp = p ** 2
+            pp = p**2
             ppp = pp * p
             p2n = p + 2 * n
 
-            N1 = -(n - 1) * f ** 2 * pp * (q - n)
-            N2a = (-8 + 8 * p + 8 * q) * n ** 3
-            N2b = (16 * pp + (-44 + 20 * q) * p + 26 - 24 * q) * n ** 2
+            N1 = -(n - 1) * f**2 * pp * (q - n)
+            N2a = (-8 + 8 * p + 8 * q) * n**3
+            N2b = (16 * pp + (-44 + 20 * q) * p + 26 - 24 * q) * n**2
             N2c = (10 * ppp + (14 * q - 46) * pp + (-40 * q + 66) * p - 28 + 24 * q) * n
-            N2d = 2 * pp ** 2 + (-13 + 3 * q) * ppp + (-14 * q + 30) * pp
+            N2d = 2 * pp**2 + (-13 + 3 * q) * ppp + (-14 * q + 30) * pp
             N2e = (-29 + 19 * q) * p + 10 - 8 * q
 
-            D1 = q ** 2 * (p2n - 3) ** 2
+            D1 = q**2 * (p2n - 3) ** 2
             D2 = (p2n - 2) ** 3 * (p2n - 1) ** 2
 
             return (N1 / D1) * (N2a + N2b + N2c + N2d + N2e) / D2
@@ -1327,7 +1356,7 @@ class BetaIncDer(ScalarOp):
                 return p * f / (q * (p + 1))
 
             p2n = p + 2 * n
-            F1 = (p ** 2 * f ** 2 / (q ** 2)) * (n - 1) * (p + n - 1) * (2 * q + p - 2)
+            F1 = (p**2 * f**2 / (q**2)) * (n - 1) * (p + n - 1) * (2 * q + p - 2)
             D1 = (p2n - 3) * (p2n - 2) ** 2 * (p2n - 1)
 
             return F1 / D1
@@ -1337,14 +1366,14 @@ class BetaIncDer(ScalarOp):
             Derivative of b_n wrt p
             """
             p2n = p + 2 * n
-            pp = p ** 2
+            pp = p**2
             q4 = 4 * q
             p4 = 4 * p
 
             F1 = (p * f / q) * (
-                (-p4 - q4 + 4) * n ** 2 + (p4 - 4 + q4 - 2 * pp) * n + pp * q
+                (-p4 - q4 + 4) * n**2 + (p4 - 4 + q4 - 2 * pp) * n + pp * q
             )
-            D1 = (p2n - 2) ** 2 * p2n ** 2
+            D1 = (p2n - 2) ** 2 * p2n**2
 
             return F1 / D1
 
@@ -1353,7 +1382,7 @@ class BetaIncDer(ScalarOp):
             Derivative of b_n wrt to q
             """
             p2n = p + 2 * n
-            return -(p ** 2 * f) / (q * (p2n - 2) * p2n)
+            return -(p**2 * f) / (q * (p2n - 2) * p2n)
 
         # Input validation
         if not (0 <= x <= 1) or p < 0 or q < 0:
@@ -1430,5 +1459,8 @@ class BetaIncDer(ScalarOp):
         )
         return np.nan
 
+    def c_code(self, *args, **kwargs):
+        raise NotImplementedError()
+
 
 betainc_der = BetaIncDer(upgrade_to_float_no_complex, name="betainc_der")
