@@ -2,8 +2,9 @@
 
 import copy
 import dataclasses
+from itertools import chain
 from sys import maxsize
-from typing import Dict, List, Optional, Tuple
+from typing import Dict, List, Optional, Tuple, cast
 
 import numpy as np
 
@@ -80,31 +81,34 @@ def remove_constants_and_unused_inputs_scan(fgraph, node):
     if not isinstance(node.op, Scan):
         return False
     op = node.op
+    op_info = op.info
     # We only need to take care of sequences and other arguments
-    st = op.n_seqs
-    st += int(sum(len(x) for x in op.tap_array[: (op.n_mit_mot + op.n_mit_sot)]))
-    st += op.n_sit_sot
-    st += op.n_shared_outs
+    st = op_info.n_seqs
+    st += int(
+        sum(len(x) for x in chain(op_info.mit_mot_in_slices, op_info.mit_sot_in_slices))
+    )
+    st += op_info.n_sit_sot
+    st += op_info.n_shared_outs
 
-    op_ins = op.inputs
-    op_outs = op.outputs
+    op_ins = op.inner_inputs
+    op_outs = op.inner_outputs
 
     # Corresponds to the initial states, which should stay untouched.
     # We put those variables aside, and put them back at the end.
-    out_stuff_inner = op_ins[op.n_seqs : st]
+    out_stuff_inner = op_ins[op_info.n_seqs : st]
 
     non_seqs = op_ins[st:]
     st = (
-        op.n_seqs
-        + op.n_mit_mot
-        + op.n_mit_sot
-        + op.n_sit_sot
-        + op.n_nit_sot
-        + op.n_shared_outs
+        op_info.n_seqs
+        + op_info.n_mit_mot
+        + op_info.n_mit_sot
+        + op_info.n_sit_sot
+        + op_info.n_nit_sot
+        + op_info.n_shared_outs
         + 1
     )
     outer_non_seqs = node.inputs[st:]
-    out_stuff_outer = node.inputs[1 + op.n_seqs : st]
+    out_stuff_outer = node.inputs[1 + op_info.n_seqs : st]
 
     # To replace constants in the outer graph by clones in the inner graph
     givens = {}
@@ -114,7 +118,7 @@ def remove_constants_and_unused_inputs_scan(fgraph, node):
     nw_outer = [node.inputs[0]]
 
     all_ins = list(graph_inputs(op_outs))
-    for idx in range(op.n_seqs):
+    for idx in range(op_info.n_seqs):
         node_inp = node.inputs[idx + 1]
         if (
             isinstance(node_inp, TensorConstant)
@@ -168,13 +172,14 @@ def remove_constants_and_unused_inputs_scan(fgraph, node):
 
     if len(nw_inner) != len(op_ins):
         op_outs = clone_replace(op_outs, replace=givens)
-        nw_info = dataclasses.replace(op.info, n_seqs=nw_n_seqs)
+        nw_info = dataclasses.replace(
+            op_info, n_seqs=nw_n_seqs, n_non_seqs=len(nw_inner_nonseq)
+        )
         nwScan = Scan(
             nw_inner,
             op_outs,
             nw_info,
             mode=op.mode,
-            as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
             # TODO: This seems questionable
@@ -200,7 +205,7 @@ def push_out_non_seq_scan(fgraph, node):
     if not isinstance(node.op, Scan):
         return False
 
-    node_inputs, node_outputs = node.op.inputs, node.op.outputs
+    node_inputs, node_outputs = node.op.inner_inputs, node.op.inner_outputs
 
     local_fgraph_topo = io_toposort(node_inputs, node_outputs)
     local_fgraph_outs_set = set(node_outputs)
@@ -339,13 +344,16 @@ def push_out_non_seq_scan(fgraph, node):
         op_outs = clone_replace(node_outputs, replace=givens)
         op_ins = node_inputs + nw_inner
 
+        new_info = dataclasses.replace(
+            op.info, n_non_seqs=op.info.n_non_seqs + len(nw_outer)
+        )
+
         # Reconstruct node
         nwScan = Scan(
             op_ins,
             op_outs,
-            op.info,
+            new_info,
             mode=op.mode,
-            as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
             # TODO: This seems questionable
@@ -407,7 +415,7 @@ def push_out_seq_scan(fgraph, node):
     if not isinstance(node.op, Scan):
         return False
 
-    node_inputs, node_outputs = node.op.inputs, node.op.outputs
+    node_inputs, node_outputs = node.op.inner_inputs, node.op.inner_outputs
 
     local_fgraph_topo = io_toposort(node_inputs, node_outputs)
     local_fgraph_outs_set = set(node_outputs)
@@ -583,7 +591,6 @@ def push_out_seq_scan(fgraph, node):
             op_outs,
             nw_info,
             mode=op.mode,
-            as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
             # TODO: This seems questionable
@@ -600,7 +607,7 @@ def push_out_seq_scan(fgraph, node):
         replacements["remove"] = [node]
         return replacements
 
-    elif not to_keep_set and not op.as_while and not op.outer_mitmot(node.inputs):
+    elif not to_keep_set and not op.info.as_while and not op.outer_mitmot(node.inputs):
         # Nothing in the inner graph should be kept
         replace_with = {}
         for out, idx in to_replace_map.items():
@@ -611,7 +618,7 @@ def push_out_seq_scan(fgraph, node):
                 if out in op.inner_mitsot_outs(ls):
                     odx = op.inner_mitsot_outs(ls).index(out)
                     inp = op.outer_mitsot(node.inputs)[odx]
-                    st = abs(np.min(op.mitsot_taps()))
+                    st = abs(np.min(op.info.mit_sot_in_slices))
                     y = set_subtensor(inp[st:], _y)
                 elif out in op.inner_sitsot_outs(ls):
                     odx = op.inner_sitsot_outs(ls).index(out)
@@ -716,13 +723,14 @@ def push_out_inner_vars(
             fgraph, old_scan_node, old_scan_args, add_as_nitsots
         )
 
+        assert isinstance(new_scan_node.op, Scan)
+
         new_scan_args = ScanArgs(
             new_scan_node.inputs,
             new_scan_node.outputs,
-            new_scan_node.op.inputs,
-            new_scan_node.op.outputs,
+            new_scan_node.op.inner_inputs,
+            new_scan_node.op.inner_outputs,
             new_scan_node.op.info,
-            new_scan_node.op.as_while,
         )
 
         new_outs = new_scan_args.outer_out_nit_sot[-len(add_as_nitsots) :]
@@ -746,6 +754,8 @@ def add_nitsot_outputs(
     new_outputs_inner,
 ) -> Tuple[Apply, Dict[Variable, Variable]]:
 
+    assert isinstance(old_scan_node.op, Scan)
+
     nb_new_outs = len(new_outputs_inner)
 
     # Create the initial values for the new nitsot outputs
@@ -758,13 +768,14 @@ def add_nitsot_outputs(
     new_scan_args.inner_out_nit_sot.extend(new_outputs_inner)
     new_scan_args.outer_in_nit_sot.extend(new_nitsots_initial_value)
 
+    assert isinstance(old_scan_node.op, Scan)
+
     # Create the `Scan` `Op` from the `ScanArgs`
     new_scan_op = Scan(
         new_scan_args.inner_inputs,
         new_scan_args.inner_outputs,
         new_scan_args.info,
         mode=old_scan_node.op.mode,
-        as_while=old_scan_node.op.as_while,
         profile=old_scan_node.op.profile,
         truncate_gradient=old_scan_node.op.truncate_gradient,
         # TODO: This seems questionable
@@ -812,7 +823,7 @@ def push_out_add_scan(fgraph, node):
     # Don't perform the optimization on `as_while` `Scan`s. Because these
     # `Scan`s don't run for a predetermined number of steps, handling them is
     # more complicated and this optimization doesn't support it at the moment.
-    if not (isinstance(node.op, Scan) and not node.op.as_while):
+    if not (isinstance(node.op, Scan) and not node.op.info.as_while):
         return False
 
     op = node.op
@@ -820,7 +831,7 @@ def push_out_add_scan(fgraph, node):
     # Use `ScanArgs` to parse the inputs and outputs of scan for ease of
     # use
     args = ScanArgs(
-        node.inputs, node.outputs, op.inputs, op.outputs, op.info, op.as_while
+        node.inputs, node.outputs, op.inner_inputs, op.inner_outputs, op.info
     )
 
     clients = {}
@@ -917,35 +928,35 @@ class ScanInplaceOptimizer(GlobalOptimizer):
 
     """
 
-    def __init__(self, typeInfer=None):
-        super().__init__()
-        self.typeInfer = typeInfer
+    alloc_ops = (Alloc, AllocEmpty)
+    """
+    Classes that represent operation that allocate new memory and that the
+    optimization should duplicate so it can operate inplace on them.
+    """
 
     def add_requirements(self, fgraph):
         fgraph.attach_feature(ReplaceValidate())
         fgraph.attach_feature(DestroyHandler())
 
-    def attempt_scan_inplace(self, fgraph, node, output_indices, alloc_ops):
+    def attempt_scan_inplace(
+        self, fgraph: FunctionGraph, node: Apply[Scan], output_indices: List[int]
+    ) -> Optional[Apply]:
         """Attempt to replace a `Scan` node by one which computes the specified outputs inplace.
 
         Parameters
         ----------
-        fgraph : FunctionGraph
+        fgraph
             Function graph in which to attempt the replacement
-        node : Apply node
+        node
             Scan node to replace by an inplace version
-        output_indices : list of integers
+        output_indices
             Indices of the outputs to attempt to compute inplace
-        alloc_ops : list of Op classes
-            Classes that represent operation that allocate new memory and
-            that the optimization should duplicate so it can operate inplace
-            on them.
         """
 
         op = node.op
 
         # inputs corresponding to sequences and n_steps
-        ls_begin = node.inputs[: 1 + op.n_seqs]
+        ls_begin = node.inputs[: 1 + op.info.n_seqs]
         ls = op.outer_mitmot(node.inputs)
         ls += op.outer_mitsot(node.inputs)
         ls += op.outer_sitsot(node.inputs)
@@ -953,14 +964,14 @@ class ScanInplaceOptimizer(GlobalOptimizer):
         ls_end += op.outer_nitsot(node.inputs)
         ls_end += op.outer_non_seqs(node.inputs)
 
-        # In `ls`, duplicate any input which has more then one client and is
+        # In `ls`, duplicate any input which has more than one client and is
         # the output of an eligible allocation op
         for i in range(len(ls)):
             inp = ls[i]
             if (
                 len(fgraph.clients[inp]) > 1
                 and inp.owner
-                and isinstance(inp.owner.op, alloc_ops)
+                and isinstance(inp.owner.op, self.alloc_ops)
             ):
                 new_lsi = inp.owner.op.make_node(*inp.owner.inputs)
 
@@ -980,24 +991,8 @@ class ScanInplaceOptimizer(GlobalOptimizer):
                 ls[idx] = deep_copy_op(ls[idx])
 
         inputs = ls_begin + ls + ls_end
-        if self.typeInfer is None:
-            typeConstructor = None
-        else:
-            typeConstructor = self.typeInfer(node)
 
-        new_op = Scan(
-            op.inputs,
-            op.outputs,
-            op.info,
-            mode=op.mode,
-            typeConstructor=typeConstructor,
-            as_while=op.as_while,
-            profile=op.profile,
-            truncate_gradient=op.truncate_gradient,
-            # TODO: This seems questionable
-            name=op.name,
-            allow_gc=op.allow_gc,
-        )
+        new_op = op.clone()
 
         destroy_map = op.destroy_map.copy()
         for out_idx in output_indices:
@@ -1007,29 +1002,38 @@ class ScanInplaceOptimizer(GlobalOptimizer):
 
         # Do not call make_node for test_value
         new_outs = new_op(*inputs, return_list=True)
+
+        assert isinstance(new_outs, list)
+
         try:
-            fgraph.replace_all_validate_remove(
+            # TODO FIXME: We need to stop using this approach (i.e. attempt
+            # in-place replacements and wait for downstream failures to revert
+            # the changes).  It prevents us from making smart, clear
+            # rewrites and it adds a lot of unnecessary overhead that
+            # involves dealing with inconsistent graphs.
+            # This whole rewrite should be a simple local rewrite, but, because
+            # of this awful approach, it can't be.
+            fgraph.replace_all_validate_remove(  # type: ignore
                 list(zip(node.outputs, new_outs)),
                 remove=[node],
                 reason="scan_make_inplace",
             )
-            return new_outs[0].owner
+            return cast(Apply[Scan], new_outs[0].owner)
         except InconsistencyError:
             # Failed moving output to be computed inplace
-            return node
+            return None
 
     def apply(self, fgraph):
 
-        alloc_ops = (Alloc, AllocEmpty)
-        nodes = fgraph.toposort()[::-1]
-        scan_nodes = [x for x in nodes if (isinstance(x.op, Scan))]
-        for scan_idx in range(len(scan_nodes)):
+        for scan_idx, original_node in enumerate(reversed(fgraph.toposort())):
+
+            if not isinstance(original_node.op, Scan):
+                continue
 
             # First attempt to make the Scan compute inplace every recurrent
             # output that seems like it could be computed inplace. If that
             # fails, go through these outputs individually, trying each of
             # them.
-            original_node = scan_nodes[scan_idx]
             op = original_node.op
             n_outs = op.info.n_mit_mot + op.info.n_mit_sot + op.info.n_sit_sot
 
@@ -1037,13 +1041,13 @@ class ScanInplaceOptimizer(GlobalOptimizer):
             # operate inplace.
             out_indices = []
             for out_idx in range(n_outs):
-                inp_idx = 1 + op.n_seqs + out_idx
+                inp_idx = 1 + op.info.n_seqs + out_idx
                 inp = original_node.inputs[inp_idx]
 
                 # If the input is from an eligible allocation node, attempt to
                 # be inplace on it, even if other nodes are modifying it
                 # inplace.
-                if inp.owner and isinstance(inp.owner.op, alloc_ops):
+                if inp.owner and isinstance(inp.owner.op, self.alloc_ops):
                     out_indices.append(out_idx)
                     continue
 
@@ -1069,16 +1073,21 @@ class ScanInplaceOptimizer(GlobalOptimizer):
                 if not input_used_inplace:
                     out_indices.append(out_idx)
 
-            node = self.attempt_scan_inplace(
-                fgraph, scan_nodes[scan_idx], out_indices, alloc_ops
-            )
+            if len(out_indices) == 0:
+                continue
 
-            if node is original_node:
+            new_node = self.attempt_scan_inplace(fgraph, original_node, out_indices)
+
+            if new_node is None:
                 # Making the scan compute all plausible recurrent outputs
-                # inplace has failed. Attempt all plausible recurrent output
+                # inplace has failed. Attempt all plausible recurrent outputs
                 # individually.
+
+                new_node = original_node
                 for pos in out_indices:
-                    node = self.attempt_scan_inplace(fgraph, node, [pos], alloc_ops)
+                    new_node = (
+                        self.attempt_scan_inplace(fgraph, new_node, [pos]) or new_node
+                    )
 
 
 def select_min(x, y):
@@ -1146,11 +1155,16 @@ def save_mem_new_scan(fgraph, node):
     # defining ``init_l`` for mit_mot sequences is a bit trickier but
     # it is safe to set it to 0
     op = node.op
-    c_outs = op.n_mit_mot + op.n_mit_sot + op.n_sit_sot + op.n_nit_sot
+    op_info = op.info
+    c_outs = (
+        op_info.n_mit_mot + op_info.n_mit_sot + op_info.n_sit_sot + op_info.n_nit_sot
+    )
 
-    init_l = [0 for x in range(op.n_mit_mot)]
-    init_l += [abs(min(v)) for v in op.tap_array[op.n_mit_mot :]]
-    init_l += [0 for x in range(op.n_nit_sot)]
+    init_l = [0 for x in range(op_info.n_mit_mot)]
+    init_l += [
+        abs(min(v)) for v in chain(op_info.mit_sot_in_slices, op_info.sit_sot_in_slices)
+    ]
+    init_l += [0 for x in range(op_info.n_nit_sot)]
     # 2. Check the clients of each output and see for how many steps
     # does scan need to run
 
@@ -1191,8 +1205,8 @@ def save_mem_new_scan(fgraph, node):
     # Note that for mit_mot outputs and shared outputs we can not change
     # the number of intermediate steps stored without affecting the
     # result of the op
-    store_steps = [0 for o in range(op.n_mit_mot)]
-    store_steps += [-1 for o in node.outputs[op.n_mit_mot : c_outs]]
+    store_steps = [0 for o in range(op_info.n_mit_mot)]
+    store_steps += [-1 for o in node.outputs[op_info.n_mit_mot : c_outs]]
     # Flag that says if an input has changed and we need to do something
     # or not
     flag_store = False
@@ -1230,7 +1244,7 @@ def save_mem_new_scan(fgraph, node):
                     break
 
                 # 2.3.2 extract the begin/end of the first dimension
-                if i >= op.n_mit_mot:
+                if i >= op_info.n_mit_mot:
                     try:
                         length = shape_of[out][0]
                     except KeyError:
@@ -1332,7 +1346,7 @@ def save_mem_new_scan(fgraph, node):
                     store_steps[i] = 0
                     break
 
-                if i > op.n_mit_mot:
+                if i > op_info.n_mit_mot:
                     length = node.inputs[0] + init_l[i]
                 else:
                     try:
@@ -1360,9 +1374,9 @@ def save_mem_new_scan(fgraph, node):
                     # the pre-allocation mechanism is activated.
                     prealloc_outs = config.scan__allow_output_prealloc
 
-                    first_mitsot_idx = node.op.n_mit_mot
+                    first_mitsot_idx = op_info.n_mit_mot
                     last_sitsot_idx = (
-                        node.op.n_mit_mot + node.op.n_mit_sot + node.op.n_sit_sot - 1
+                        op_info.n_mit_mot + op_info.n_mit_sot + op_info.n_sit_sot - 1
                     )
                     preallocable_output = first_mitsot_idx <= i <= last_sitsot_idx
 
@@ -1394,18 +1408,18 @@ def save_mem_new_scan(fgraph, node):
         # to store everything in memory ( or ar orphane and required
         # by the inner function .. )
         replaced_outs = []
-        offset = 1 + op.n_seqs + op.n_mit_mot
-        for idx, _val in enumerate(store_steps[op.n_mit_mot :]):
-            i = idx + op.n_mit_mot
+        offset = 1 + op_info.n_seqs + op_info.n_mit_mot
+        for idx, _val in enumerate(store_steps[op_info.n_mit_mot :]):
+            i = idx + op_info.n_mit_mot
             if not (isinstance(_val, int) and _val <= 0 and i not in required):
 
-                if idx + op.n_mit_mot in required:
+                if idx + op_info.n_mit_mot in required:
                     val = 1
                 else:
                     val = _val
                 # If the memory for this output has been pre-allocated
                 # before going into the scan op (by an alloc node)
-                if idx < op.n_mit_sot + op.n_sit_sot:
+                if idx < op_info.n_mit_sot + op_info.n_sit_sot:
                     # In case the input is still an alloc node, we
                     # actually have two options:
                     #   a) the input is a set_subtensor, in that case we
@@ -1435,8 +1449,8 @@ def save_mem_new_scan(fgraph, node):
                         nw_input = nw_inputs[offset + idx][:tmp]
 
                     nw_inputs[offset + idx] = nw_input
-                    replaced_outs.append(op.n_mit_mot + idx)
-                    odx = op.n_mit_mot + idx
+                    replaced_outs.append(op_info.n_mit_mot + idx)
+                    odx = op_info.n_mit_mot + idx
                     old_outputs += [
                         (
                             odx,
@@ -1447,12 +1461,18 @@ def save_mem_new_scan(fgraph, node):
                         )
                     ]
                 # If there is no memory pre-allocated for this output
-                elif idx < op.n_mit_sot + op.n_sit_sot + op.n_nit_sot:
+                elif idx < op_info.n_mit_sot + op_info.n_sit_sot + op_info.n_nit_sot:
 
-                    pos = op.n_mit_mot + idx + op.n_seqs + 1 + op.n_shared_outs
+                    pos = (
+                        op_info.n_mit_mot
+                        + idx
+                        + op_info.n_seqs
+                        + 1
+                        + op_info.n_shared_outs
+                    )
                     if nw_inputs[pos] == node.inputs[0]:
                         nw_inputs[pos] = val
-                    odx = op.n_mit_mot + idx
+                    odx = op_info.n_mit_mot + idx
                     replaced_outs.append(odx)
                     old_outputs += [
                         (
@@ -1466,14 +1486,14 @@ def save_mem_new_scan(fgraph, node):
         # 3.4. Recompute inputs for everything else based on the new
         # number of steps
         if global_nsteps is not None:
-            for idx, val in enumerate(store_steps[op.n_mit_mot :]):
+            for idx, val in enumerate(store_steps[op_info.n_mit_mot :]):
                 if val == 0:
                     # val == 0 means that we want to keep all intermediate
                     # results for that state, including the initial values.
-                    if idx < op.n_mit_sot + op.n_sit_sot:
+                    if idx < op_info.n_mit_sot + op_info.n_sit_sot:
                         in_idx = offset + idx
                         # Number of steps in the initial state
-                        initl = init_l[op.n_mit_mot + idx]
+                        initl = init_l[op_info.n_mit_mot + idx]
 
                         # If the initial buffer has the form
                         # inc_subtensor(zeros(...)[...], _nw_input)
@@ -1494,8 +1514,10 @@ def save_mem_new_scan(fgraph, node):
                         else:
                             nw_input = nw_inputs[in_idx][: (initl + nw_steps)]
 
-                    elif idx < op.n_mit_sot + op.n_sit_sot + op.n_nit_sot:
-                        in_idx = offset + idx + op.n_shared_outs
+                    elif (
+                        idx < op_info.n_mit_sot + op_info.n_sit_sot + op_info.n_nit_sot
+                    ):
+                        in_idx = offset + idx + op_info.n_shared_outs
                         if nw_inputs[in_idx] == node.inputs[0]:
                             nw_inputs[in_idx] = nw_steps
 
@@ -1519,7 +1541,6 @@ def save_mem_new_scan(fgraph, node):
             outs,
             info,
             mode=op.mode,
-            as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
             # TODO: This seems questionable
@@ -1656,9 +1677,9 @@ class ScanMerge(GlobalOptimizer):
 
     def merge(self, nodes):
 
-        if nodes[0].op.as_while:
+        if nodes[0].op.info.as_while:
             as_while = True
-            condition = nodes[0].op.outputs[-1]
+            condition = nodes[0].op.inner_outputs[-1]
         else:
             as_while = False
 
@@ -1678,56 +1699,58 @@ class ScanMerge(GlobalOptimizer):
             return ls
 
         for idx, nd in enumerate(nodes):
-            inner_ins[idx].append(rename(nd.op.inner_seqs(nd.op.inputs), idx))
+            inner_ins[idx].append(rename(nd.op.inner_seqs(nd.op.inner_inputs), idx))
             outer_ins += rename(nd.op.outer_seqs(nd.inputs), idx)
 
-        tap_array = ()
         mit_mot_out_slices = ()
 
+        mit_mot_in_slices = ()
         for idx, nd in enumerate(nodes):
-            # MitMot
-            inner_ins[idx].append(rename(nd.op.inner_mitmot(nd.op.inputs), idx))
-            inner_outs[idx].append(nd.op.inner_mitmot_outs(nd.op.outputs))
-            tap_array += nd.op.mitmot_taps()
-            mit_mot_out_slices += nd.op.mitmot_out_taps()
+            inner_ins[idx].append(rename(nd.op.inner_mitmot(nd.op.inner_inputs), idx))
+            inner_outs[idx].append(nd.op.inner_mitmot_outs(nd.op.inner_outputs))
+            mit_mot_in_slices += nd.op.info.mit_mot_in_slices
+            mit_mot_out_slices += nd.op.info.mit_mot_out_slices[: nd.op.info.n_mit_mot]
             outer_ins += rename(nd.op.outer_mitmot(nd.inputs), idx)
             outer_outs += nd.op.outer_mitmot_outs(nd.outputs)
 
+        mit_sot_in_slices = ()
         for idx, nd in enumerate(nodes):
-            # MitSot
-            inner_ins[idx].append(rename(nd.op.inner_mitsot(nd.op.inputs), idx))
-            inner_outs[idx].append(nd.op.inner_mitsot_outs(nd.op.outputs))
-            tap_array += nd.op.mitsot_taps()
+            inner_ins[idx].append(rename(nd.op.inner_mitsot(nd.op.inner_inputs), idx))
+            inner_outs[idx].append(nd.op.inner_mitsot_outs(nd.op.inner_outputs))
+            mit_sot_in_slices += nd.op.info.mit_sot_in_slices
             outer_ins += rename(nd.op.outer_mitsot(nd.inputs), idx)
             outer_outs += nd.op.outer_mitsot_outs(nd.outputs)
 
+        sit_sot_in_slices = ()
         for idx, nd in enumerate(nodes):
-            # SitSot
-            inner_ins[idx].append(rename(nd.op.inner_sitsot(nd.op.inputs), idx))
-            tap_array += tuple((-1,) for x in range(nd.op.n_sit_sot))
-            inner_outs[idx].append(nd.op.inner_sitsot_outs(nd.op.outputs))
+            inner_ins[idx].append(rename(nd.op.inner_sitsot(nd.op.inner_inputs), idx))
+            sit_sot_in_slices += tuple((-1,) for x in range(nd.op.info.n_sit_sot))
+            inner_outs[idx].append(nd.op.inner_sitsot_outs(nd.op.inner_outputs))
             outer_ins += rename(nd.op.outer_sitsot(nd.inputs), idx)
             outer_outs += nd.op.outer_sitsot_outs(nd.outputs)
 
         for idx, nd in enumerate(nodes):
             # Shared
-            inner_ins[idx].append(rename(nd.op.inner_shared(nd.op.inputs), idx))
+            inner_ins[idx].append(rename(nd.op.inner_shared(nd.op.inner_inputs), idx))
             outer_ins += rename(nd.op.outer_shared(nd.inputs), idx)
 
         for idx, nd in enumerate(nodes):
             # NitSot
-            inner_outs[idx].append(nd.op.inner_nitsot_outs(nd.op.outputs))
+            inner_outs[idx].append(nd.op.inner_nitsot_outs(nd.op.inner_outputs))
             outer_ins += rename(nd.op.outer_nitsot(nd.inputs), idx)
             outer_outs += nd.op.outer_nitsot_outs(nd.outputs)
 
         for idx, nd in enumerate(nodes):
             # Shared
             outer_outs += nd.op.outer_shared_outs(nd.outputs)
-            inner_outs[idx].append(nd.op.inner_shared_outs(nd.op.outputs))
+            inner_outs[idx].append(nd.op.inner_shared_outs(nd.op.inner_outputs))
 
+        n_non_seqs = 0
         for idx, nd in enumerate(nodes):
             # Non Seqs
-            inner_ins[idx].append(rename(nd.op.inner_non_seqs(nd.op.inputs), idx))
+            node_inner_non_seqs = nd.op.inner_non_seqs(nd.op.inner_inputs)
+            n_non_seqs += len(node_inner_non_seqs)
+            inner_ins[idx].append(rename(node_inner_non_seqs, idx))
             outer_ins += rename(nd.op.outer_non_seqs(nd.inputs), idx)
 
         # Add back the number of steps
@@ -1794,15 +1817,15 @@ class ScanMerge(GlobalOptimizer):
                     new_inner_outs += inner_outs[idx][gr_idx]
 
         info = ScanInfo(
-            tap_array=tap_array,
-            n_seqs=sum(nd.op.n_seqs for nd in nodes),
-            n_mit_mot=sum(nd.op.n_mit_mot for nd in nodes),
-            n_mit_mot_outs=sum(nd.op.n_mit_mot_outs for nd in nodes),
+            n_seqs=sum(nd.op.info.n_seqs for nd in nodes),
+            mit_mot_in_slices=mit_mot_in_slices,
             mit_mot_out_slices=mit_mot_out_slices,
-            n_mit_sot=sum(nd.op.n_mit_sot for nd in nodes),
-            n_sit_sot=sum(nd.op.n_sit_sot for nd in nodes),
-            n_shared_outs=sum(nd.op.n_shared_outs for nd in nodes),
-            n_nit_sot=sum(nd.op.n_nit_sot for nd in nodes),
+            mit_sot_in_slices=mit_sot_in_slices,
+            sit_sot_in_slices=sit_sot_in_slices,
+            n_nit_sot=sum(nd.op.info.n_nit_sot for nd in nodes),
+            n_shared_outs=sum(nd.op.info.n_shared_outs for nd in nodes),
+            n_non_seqs=n_non_seqs,
+            as_while=as_while,
         )
 
         old_op = nodes[0].op
@@ -1815,7 +1838,6 @@ class ScanMerge(GlobalOptimizer):
             truncate_gradient=old_op.truncate_gradient,
             allow_gc=old_op.allow_gc,
             name="&".join([nd.op.name for nd in nodes]),
-            as_while=as_while,
         )
         new_outs = new_op(*outer_ins)
 
@@ -1836,7 +1858,7 @@ class ScanMerge(GlobalOptimizer):
         """
         rep = set_nodes[0]
         if (
-            rep.op.as_while != node.op.as_while
+            rep.op.info.as_while != node.op.info.as_while
             or node.op.truncate_gradient != rep.op.truncate_gradient
             or node.op.mode != rep.op.mode
         ):
@@ -1862,11 +1884,13 @@ class ScanMerge(GlobalOptimizer):
             if is_in_ancestors(node, nd) or is_in_ancestors(nd, node):
                 return False
 
-        if not node.op.as_while:
+        if not node.op.info.as_while:
             return True
-        cond = node.op.outputs[-1]
-        rep_cond = rep.op.outputs[-1]
-        return equal_computations([cond], [rep_cond], node.op.inputs, rep.op.inputs)
+        cond = node.op.inner_outputs[-1]
+        rep_cond = rep.op.inner_outputs[-1]
+        return equal_computations(
+            [cond], [rep_cond], node.op.inner_inputs, rep.op.inner_inputs
+        )
 
     def apply(self, fgraph):
         # Collect all scan nodes ordered according to toposort
@@ -1944,10 +1968,9 @@ def scan_merge_inouts(fgraph, node):
     a = ScanArgs(
         node.inputs,
         node.outputs,
-        node.op.inputs,
-        node.op.outputs,
+        node.op.inner_inputs,
+        node.op.inner_outputs,
         node.op.info,
-        node.op.as_while,
     )
 
     inp_equiv = {}
@@ -1991,7 +2014,6 @@ def scan_merge_inouts(fgraph, node):
             inner_outputs,
             info,
             mode=node.op.mode,
-            as_while=node.op.as_while,
             profile=node.op.profile,
             truncate_gradient=node.op.truncate_gradient,
             # TODO: This seems questionable
@@ -2006,10 +2028,9 @@ def scan_merge_inouts(fgraph, node):
         na = ScanArgs(
             outer_inputs,
             outputs,
-            new_op.inputs,
-            new_op.outputs,
+            new_op.inner_inputs,
+            new_op.inner_outputs,
             new_op.info,
-            new_op.as_while,
         )
         remove = [node]
     else:
@@ -2150,10 +2171,10 @@ def push_out_dot1_scan(fgraph, node):
     # Note that this works when only you need X[-1] in the end
     # and assumes dimshuffle are applied to vectors before calling dot
     op = node.op
-    sitsot_ins = op.inner_sitsot(op.inputs)
-    sitsot_outs = op.inner_sitsot_outs(op.outputs)
+    sitsot_ins = op.inner_sitsot(op.inner_inputs)
+    sitsot_outs = op.inner_sitsot_outs(op.inner_outputs)
     outer_sitsot = op.outer_sitsot_outs(node.outputs)
-    seqs = op.inner_seqs(op.inputs)
+    seqs = op.inner_seqs(op.inner_inputs)
     for inp, out, outer_out in zip(sitsot_ins, sitsot_outs, outer_sitsot):
 
         if (
@@ -2195,34 +2216,29 @@ def push_out_dot1_scan(fgraph, node):
                     # First let us split all arguments according to their
                     # corresponding categories
 
-                    inner_seqs = op.inner_seqs(op.inputs)
+                    inner_seqs = op.inner_seqs(op.inner_inputs)
                     outer_seqs = op.outer_seqs(node.inputs)
-                    inner_mitmot = op.inner_mitmot(op.inputs)
+                    inner_mitmot = op.inner_mitmot(op.inner_inputs)
                     outer_mitmot = op.outer_mitmot(node.inputs)
-                    inner_mitmot_outs = op.inner_mitmot_outs(op.outputs)
-                    inner_mitsot = op.inner_mitsot(op.inputs)
+                    inner_mitmot_outs = op.inner_mitmot_outs(op.inner_outputs)
+                    inner_mitsot = op.inner_mitsot(op.inner_inputs)
                     outer_mitsot = op.outer_mitsot(node.inputs)
-                    inner_mitsot_outs = op.inner_mitsot_outs(op.outputs)
-                    inner_sitsot = op.inner_sitsot(op.inputs)
+                    inner_mitsot_outs = op.inner_mitsot_outs(op.inner_outputs)
+                    inner_sitsot = op.inner_sitsot(op.inner_inputs)
                     outer_sitsot = op.outer_sitsot(node.inputs)
-                    inner_sitsot_outs = op.inner_sitsot_outs(op.outputs)
+                    inner_sitsot_outs = op.inner_sitsot_outs(op.inner_outputs)
                     outer_nitsot = op.outer_nitsot(node.inputs)
-                    inner_nitsot_outs = op.inner_nitsot_outs(op.outputs)
-                    inner_shared = op.inner_shared(op.inputs)
+                    inner_nitsot_outs = op.inner_nitsot_outs(op.inner_outputs)
+                    inner_shared = op.inner_shared(op.inner_inputs)
                     outer_shared = op.outer_shared(node.inputs)
-                    inner_shared_outs = op.inner_shared_outs(op.outputs)
-                    inner_non_seqs = op.inner_non_seqs(op.inputs)
+                    inner_shared_outs = op.inner_shared_outs(op.inner_outputs)
+                    inner_non_seqs = op.inner_non_seqs(op.inner_inputs)
                     outer_non_seqs = op.outer_non_seqs(node.inputs)
 
-                    st = len(op.mitmot_taps()) + len(op.mitsot_taps())
-
                     new_info = dataclasses.replace(
                         op.info,
-                        tap_array=(
-                            op.info.tap_array[: st + idx]
-                            + op.info.tap_array[st + idx + 1 :]
-                        ),
-                        n_sit_sot=op.info.n_sit_sot - 1,
+                        sit_sot_in_slices=op.info.sit_sot_in_slices[:idx]
+                        + op.info.sit_sot_in_slices[idx + 1 :],
                         n_nit_sot=op.info.n_nit_sot + 1,
                     )
                     inner_sitsot = inner_sitsot[:idx] + inner_sitsot[idx + 1 :]
@@ -2256,7 +2272,6 @@ def push_out_dot1_scan(fgraph, node):
                         new_inner_outs,
                         new_info,
                         mode=op.mode,
-                        as_while=op.as_while,
                         profile=op.profile,
                         truncate_gradient=op.truncate_gradient,
                         # TODO: This seems questionable
@@ -2351,7 +2366,7 @@ optdb.register(
 )
 optdb.register(
     "scan_make_inplace",
-    ScanInplaceOptimizer(typeInfer=None),
+    ScanInplaceOptimizer(),
     "fast_run",
     "inplace",
     "scan",
