@@ -4,6 +4,7 @@ import copy
 import dataclasses
 import logging
 from collections import OrderedDict, namedtuple
+from itertools import chain
 from typing import TYPE_CHECKING, Callable, List, Optional, Sequence, Set, Tuple, Union
 from typing import cast as type_cast
 
@@ -348,25 +349,31 @@ class Validator:
 
 
 def scan_can_remove_outs(op, out_idxs):
-    """
-    Looks at all outputs defined by indices ``out_idxs`` and see whom can be
-    removed from the scan op without affecting the rest. Return two lists,
-    the first one with the indices of outs that can be removed, the second
-    with the outputs that can not be removed.
+    """Look at all outputs defined by indices ``out_idxs`` and determines which can be removed.
+
+    Returns
+    -------
+    two lists, the first one with the indices of outs that can be removed, the
+    second with the outputs that can not be removed.
 
     """
-    non_removable = [o for i, o in enumerate(op.outputs) if i not in out_idxs]
+    non_removable = [o for i, o in enumerate(op.inner_outputs) if i not in out_idxs]
     required_inputs = list(graph_inputs(non_removable))
 
     out_ins = []
-    offset = op.n_seqs
-    lim = op.n_mit_mot + op.n_mit_sot + op.n_sit_sot
-    for idx in range(lim):
-        n_ins = len(op.info.tap_array[idx])
-        out_ins += [op.inputs[offset : offset + n_ins]]
+    offset = op.info.n_seqs
+    for idx, tap in enumerate(
+        chain(
+            op.info.mit_mot_in_slices,
+            op.info.mit_sot_in_slices,
+            op.info.sit_sot_in_slices,
+        )
+    ):
+        n_ins = len(tap)
+        out_ins += [op.inner_inputs[offset : offset + n_ins]]
         offset += n_ins
-    out_ins += [[] for k in range(op.n_nit_sot)]
-    out_ins += [[op.inputs[offset + k]] for k in range(op.n_shared_outs)]
+    out_ins += [[] for k in range(op.info.n_nit_sot)]
+    out_ins += [[op.inner_inputs[offset + k]] for k in range(op.info.n_shared_outs)]
 
     added = True
     out_idxs_mask = [1 for idx in out_idxs]
@@ -376,7 +383,7 @@ def scan_can_remove_outs(op, out_idxs):
             if out_idxs_mask[pos] and any(x in required_inputs for x in out_ins[idx]):
                 # This output is required ..
                 out_idxs_mask[pos] = 0
-                required_inputs += list(graph_inputs([op.outputs[idx]]))
+                required_inputs += list(graph_inputs([op.inner_outputs[idx]]))
                 added = True
 
     required_outs = [x for i, x in enumerate(out_idxs) if out_idxs_mask[i] == 0]
@@ -397,94 +404,95 @@ def compress_outs(op, not_required, inputs):
     """
     from aesara.scan.op import ScanInfo
 
+    op_info = op.info
     info = ScanInfo(
-        tap_array=(),
-        n_seqs=op.info.n_seqs,
-        n_mit_mot=0,
-        n_mit_mot_outs=0,
+        n_seqs=op_info.n_seqs,
+        mit_mot_in_slices=(),
         mit_mot_out_slices=(),
-        n_mit_sot=0,
-        n_sit_sot=0,
-        n_shared_outs=0,
+        mit_sot_in_slices=(),
+        sit_sot_in_slices=(),
         n_nit_sot=0,
+        n_shared_outs=0,
+        n_non_seqs=0,
+        as_while=op_info.as_while,
     )
 
-    op_inputs = op.inputs[: op.n_seqs]
+    op_inputs = op.inner_inputs[: op_info.n_seqs]
     op_outputs = []
-    node_inputs = inputs[: op.n_seqs + 1]
+    node_inputs = inputs[: op_info.n_seqs + 1]
     map_old_new = OrderedDict()
 
     offset = 0
-    ni_offset = op.n_seqs + 1
-    i_offset = op.n_seqs
+    ni_offset = op_info.n_seqs + 1
+    i_offset = op_info.n_seqs
     o_offset = 0
     curr_pos = 0
-    for idx in range(op.info.n_mit_mot):
+    for idx in range(op_info.n_mit_mot):
         if offset + idx not in not_required:
             map_old_new[offset + idx] = curr_pos
             curr_pos += 1
             info = dataclasses.replace(
                 info,
-                n_mit_mot=info.n_mit_mot + 1,
-                tap_array=info.tap_array + (tuple(op.tap_array[offset + idx]),),
+                mit_mot_in_slices=info.mit_mot_in_slices
+                + (op_info.mit_mot_in_slices[idx],),
                 mit_mot_out_slices=info.mit_mot_out_slices
-                + (tuple(op.mit_mot_out_slices[offset + idx]),),
+                + (op_info.mit_mot_out_slices[idx],),
             )
             # input taps
-            for jdx in op.tap_array[offset + idx]:
-                op_inputs += [op.inputs[i_offset]]
+            for jdx in op_info.mit_mot_in_slices[idx]:
+                op_inputs += [op.inner_inputs[i_offset]]
                 i_offset += 1
             # output taps
-            for jdx in op.mit_mot_out_slices[offset + idx]:
-                op_outputs += [op.outputs[o_offset]]
+            for jdx in op_info.mit_mot_out_slices[idx]:
+                op_outputs += [op.inner_outputs[o_offset]]
                 o_offset += 1
             # node inputs
             node_inputs += [inputs[ni_offset + idx]]
         else:
-            o_offset += len(op.mit_mot_out_slices[offset + idx])
-            i_offset += len(op.tap_array[offset + idx])
-    info = dataclasses.replace(info, n_mit_mot_outs=len(op_outputs))
-    offset += op.n_mit_mot
-    ni_offset += op.n_mit_mot
+            o_offset += len(op_info.mit_mot_out_slices[idx])
+            i_offset += len(op_info.mit_mot_in_slices[idx])
 
-    for idx in range(op.info.n_mit_sot):
+    offset += op_info.n_mit_mot
+    ni_offset += op_info.n_mit_mot
+
+    for idx in range(op_info.n_mit_sot):
         if offset + idx not in not_required:
             map_old_new[offset + idx] = curr_pos
             curr_pos += 1
             info = dataclasses.replace(
                 info,
-                n_mit_sot=info.n_mit_sot + 1,
-                tap_array=info.tap_array + (tuple(op.tap_array[offset + idx]),),
+                mit_sot_in_slices=info.mit_sot_in_slices
+                + (op_info.mit_sot_in_slices[idx],),
             )
             # input taps
-            for jdx in op.tap_array[offset + idx]:
-                op_inputs += [op.inputs[i_offset]]
+            for jdx in op_info.mit_sot_in_slices[idx]:
+                op_inputs += [op.inner_inputs[i_offset]]
                 i_offset += 1
             # output taps
-            op_outputs += [op.outputs[o_offset]]
+            op_outputs += [op.inner_outputs[o_offset]]
             o_offset += 1
             # node inputs
             node_inputs += [inputs[ni_offset + idx]]
         else:
             o_offset += 1
-            i_offset += len(op.tap_array[offset + idx])
+            i_offset += len(op_info.mit_sot_in_slices[idx])
 
-    offset += op.n_mit_sot
-    ni_offset += op.n_mit_sot
-    for idx in range(op.info.n_sit_sot):
+    offset += op_info.n_mit_sot
+    ni_offset += op_info.n_mit_sot
+    for idx in range(op_info.n_sit_sot):
         if offset + idx not in not_required:
             map_old_new[offset + idx] = curr_pos
             curr_pos += 1
             info = dataclasses.replace(
                 info,
-                n_sit_sot=info.n_sit_sot + 1,
-                tap_array=info.tap_array + (tuple(op.tap_array[offset + idx]),),
+                sit_sot_in_slices=info.sit_sot_in_slices
+                + (op_info.sit_sot_in_slices[idx],),
             )
             # input taps
-            op_inputs += [op.inputs[i_offset]]
+            op_inputs += [op.inner_inputs[i_offset]]
             i_offset += 1
             # output taps
-            op_outputs += [op.outputs[o_offset]]
+            op_outputs += [op.inner_outputs[o_offset]]
             o_offset += 1
             # node inputs
             node_inputs += [inputs[ni_offset + idx]]
@@ -492,30 +500,30 @@ def compress_outs(op, not_required, inputs):
             o_offset += 1
             i_offset += 1
 
-    offset += op.n_sit_sot
-    ni_offset += op.n_sit_sot
+    offset += op_info.n_sit_sot
+    ni_offset += op_info.n_sit_sot
     nit_sot_ins = []
-    for idx in range(op.info.n_nit_sot):
+    for idx in range(op_info.n_nit_sot):
         if offset + idx not in not_required:
             map_old_new[offset + idx] = curr_pos
             curr_pos += 1
             info = dataclasses.replace(info, n_nit_sot=info.n_nit_sot + 1)
-            op_outputs += [op.outputs[o_offset]]
+            op_outputs += [op.inner_outputs[o_offset]]
             o_offset += 1
-            nit_sot_ins += [inputs[ni_offset + idx + op.n_shared_outs]]
+            nit_sot_ins += [inputs[ni_offset + idx + op_info.n_shared_outs]]
         else:
             o_offset += 1
 
-    offset += op.n_nit_sot
+    offset += op_info.n_nit_sot
     shared_ins = []
-    for idx in range(op.info.n_shared_outs):
+    for idx in range(op_info.n_shared_outs):
         if offset + idx not in not_required:
             map_old_new[offset + idx] = curr_pos
             curr_pos += 1
             info = dataclasses.replace(info, n_shared_outs=info.n_shared_outs + 1)
-            op_outputs += [op.outputs[o_offset]]
+            op_outputs += [op.inner_outputs[o_offset]]
             o_offset += 1
-            op_inputs += [op.inputs[i_offset]]
+            op_inputs += [op.inner_inputs[i_offset]]
             i_offset += 1
             shared_ins += [inputs[ni_offset + idx]]
         else:
@@ -524,10 +532,11 @@ def compress_outs(op, not_required, inputs):
     node_inputs += shared_ins
     node_inputs += nit_sot_ins
     # other stuff
-    op_inputs += op.inputs[i_offset:]
-    node_inputs += inputs[ni_offset + op.n_shared_outs + op.n_nit_sot :]
-    if op.as_while:
-        op_outputs += [op.outputs[o_offset]]
+    op_inputs += op.inner_inputs[i_offset:]
+    info = dataclasses.replace(info, n_non_seqs=len(op.inner_inputs[i_offset:]))
+    node_inputs += inputs[ni_offset + op_info.n_shared_outs + op_info.n_nit_sot :]
+    if op_info.as_while:
+        op_outputs += [op.inner_outputs[o_offset]]
         map_old_new[o_offset] = len(op_outputs) - 1
         # map_old_new[len(op_outputs)-1] = o_offset
 
@@ -580,11 +589,10 @@ class ScanArgs:
         _inner_inputs: Sequence[Variable],
         _inner_outputs: Sequence[Variable],
         info: "ScanInfo",
-        as_while: bool,
         clone: Optional[bool] = True,
     ):
         self.n_steps = outer_inputs[0]
-        self.as_while = as_while
+        self.as_while = info.as_while
 
         if clone:
             rval = reconstruct_graph(_inner_inputs, _inner_outputs, "")
@@ -611,8 +619,9 @@ class ScanArgs:
         n_mit_mot = info.n_mit_mot
         n_mit_sot = info.n_mit_sot
 
-        self.mit_mot_in_slices = list(info.tap_array[:n_mit_mot])
-        self.mit_sot_in_slices = list(info.tap_array[n_mit_mot : n_mit_mot + n_mit_sot])
+        self.mit_mot_in_slices = info.mit_mot_in_slices
+        self.mit_sot_in_slices = info.mit_sot_in_slices
+        self.sit_sot_in_slices = info.sit_sot_in_slices
 
         n_mit_mot_ins = sum(len(s) for s in self.mit_mot_in_slices)
         n_mit_sot_ins = sum(len(s) for s in self.mit_sot_in_slices)
@@ -705,10 +714,9 @@ class ScanArgs:
         return ScanArgs(
             node.inputs,
             node.outputs,
-            node.op.inputs,
-            node.op.outputs,
+            node.op.inner_inputs,
+            node.op.inner_outputs,
             node.op.info,
-            node.op.as_while,
             clone=clone,
         )
 
@@ -800,18 +808,14 @@ class ScanArgs:
 
         return ScanInfo(
             n_seqs=len(self.outer_in_seqs),
-            n_mit_mot=len(self.outer_in_mit_mot),
-            n_mit_sot=len(self.outer_in_mit_sot),
-            tap_array=(
-                tuple(tuple(v) for v in self.mit_mot_in_slices)
-                + tuple(tuple(v) for v in self.mit_sot_in_slices)
-                + ((-1,),) * len(self.inner_in_sit_sot)
-            ),
-            n_sit_sot=len(self.outer_in_sit_sot),
+            mit_mot_in_slices=tuple(tuple(v) for v in self.mit_mot_in_slices),
+            mit_mot_out_slices=tuple(self.mit_mot_out_slices),
+            mit_sot_in_slices=tuple(tuple(v) for v in self.mit_sot_in_slices),
+            sit_sot_in_slices=((-1,),) * len(self.inner_in_sit_sot),
             n_nit_sot=len(self.outer_in_nit_sot),
             n_shared_outs=len(self.outer_in_shared),
-            n_mit_mot_outs=sum(len(s) for s in self.mit_mot_out_slices),
-            mit_mot_out_slices=tuple(self.mit_mot_out_slices),
+            n_non_seqs=len(self.inner_in_non_seqs),
+            as_while=self.as_while,
         )
 
     def get_alt_field(
