@@ -8,7 +8,7 @@ from aesara import scalar as aes
 from aesara.gradient import DisconnectedType
 from aesara.graph.basic import Apply
 from aesara.graph.op import Op
-from aesara.tensor import basic as aet
+from aesara.tensor import basic as at
 from aesara.tensor import math as tm
 from aesara.tensor.basic import as_tensor_variable, extract_diag
 from aesara.tensor.type import dvector, lscalar, matrix, scalar, vector
@@ -54,8 +54,8 @@ class MatrixPinv(Op):
 
         grad = (
             -matrix_dot(z, gz.T, z)
-            + matrix_dot(z, z.T, gz, (aet.identity_like(x_dot_z) - x_dot_z))
-            + matrix_dot((aet.identity_like(z_dot_x) - z_dot_x), gz, z.T, z)
+            + matrix_dot(z, z.T, gz, (at.identity_like(x_dot_z) - x_dot_z))
+            + matrix_dot((at.identity_like(z_dot_x) - z_dot_x), gz, z.T, z)
         ).T
         return [grad]
 
@@ -193,11 +193,6 @@ def matrix_dot(*args):
 def trace(X):
     """
     Returns the sum of diagonal elements of matrix X.
-
-    Notes
-    -----
-    Works on GPU since 0.6rc4.
-
     """
     return extract_diag(X).sum()
 
@@ -279,7 +274,7 @@ class Eigh(Eig):
     __props__ = ("UPLO",)
 
     def __init__(self, UPLO="L"):
-        assert UPLO in ["L", "U"]
+        assert UPLO in ("L", "U")
         self.UPLO = UPLO
 
     def make_node(self, x):
@@ -350,7 +345,7 @@ class EighGrad(Op):
     __props__ = ("UPLO",)
 
     def __init__(self, UPLO="L"):
-        assert UPLO in ["L", "U"]
+        assert UPLO in ("L", "U")
         self.UPLO = UPLO
         if UPLO == "L":
             self.tri0 = np.tril
@@ -641,7 +636,7 @@ def matrix_power(M, n):
 
     # Shortcuts when 0 < n <= 3
     if n == 0:
-        return aet.eye(M.shape[-2])
+        return at.eye(M.shape[-2])
 
     elif n == 1:
         return M
@@ -670,7 +665,7 @@ def norm(x, ord):
         raise ValueError("'axis' entry is out of bounds.")
     elif ndim == 1:
         if ord is None:
-            return tm.sum(x ** 2) ** 0.5
+            return tm.sum(x**2) ** 0.5
         elif ord == "inf":
             return tm.max(abs(x))
         elif ord == "-inf":
@@ -679,13 +674,13 @@ def norm(x, ord):
             return x[x.nonzero()].shape[0]
         else:
             try:
-                z = tm.sum(abs(x ** ord)) ** (1.0 / ord)
+                z = tm.sum(abs(x**ord)) ** (1.0 / ord)
             except TypeError:
                 raise ValueError("Invalid norm order for vectors.")
             return z
     elif ndim == 2:
         if ord is None or ord == "fro":
-            return tm.sum(abs(x ** 2)) ** (0.5)
+            return tm.sum(abs(x**2)) ** (0.5)
         elif ord == "inf":
             return tm.max(tm.sum(abs(x), 1))
         elif ord == "-inf":
@@ -729,7 +724,6 @@ class TensorInv(Op):
 
 def tensorinv(a, ind=2):
     """
-    Does not run on GPU;
     Aesara utilization of numpy.linalg.tensorinv;
 
     Compute the 'inverse' of an N-dimensional array.
@@ -791,7 +785,7 @@ class TensorSolve(Op):
 
 def tensorsolve(a, b, axes=None):
     """
-    Aesara utilization of numpy.linalg.tensorsolve. Does not run on GPU!
+    Aesara utilization of numpy.linalg.tensorsolve.
 
     Solve the tensor equation ``a x = b`` for x.
     It is assumed that all indices of `x` are summed over in the product,
