@@ -5,9 +5,10 @@ from typing import Optional
 import aesara
 from aesara.configdefaults import config
 from aesara.graph.basic import Apply
-from aesara.graph.op import OpenMPOp, _NoPythonOp
-from aesara.graph.params_type import ParamsType
-from aesara.graph.type import EnumList
+from aesara.graph.op import _NoPythonOp
+from aesara.link.c.op import OpenMPOp
+from aesara.link.c.params_type import ParamsType
+from aesara.link.c.type import EnumList
 from aesara.scalar import int8, int64
 from aesara.tensor import blas_headers
 from aesara.tensor.basic import as_tensor_variable
@@ -138,7 +139,7 @@ class BaseCorrMM(OpenMPOp, _NoPythonOp):
             else:
                 self.blas_type = ""
 
-        if self._direction not in ["forward", "backprop weights", "backprop inputs"]:
+        if self._direction not in ("forward", "backprop weights", "backprop inputs"):
             raise ValueError(
                 "_direction must be one of 'forward', "
                 "'backprop weights', 'backprop inputs'"
