@@ -14,7 +14,6 @@ from aesara.tensor.basic import (
     ARange,
     Join,
     MakeVector,
-    Rebroadcast,
     ScalarFromTensor,
     TensorFromScalar,
     alloc,
@@ -23,7 +22,6 @@ from aesara.tensor.basic import (
     concatenate,
     extract_constant,
     get_scalar_constant_value,
-    patternbroadcast,
     switch,
 )
 from aesara.tensor.basic_opt import (
@@ -51,9 +49,11 @@ from aesara.tensor.math import (
 from aesara.tensor.shape import (
     Shape,
     SpecifyShape,
+    Unbroadcast,
     shape_padleft,
     shape_tuple,
     specify_shape,
+    unbroadcast,
 )
 from aesara.tensor.sharedvar import TensorSharedVariable
 from aesara.tensor.subtensor import (
@@ -371,7 +371,7 @@ def local_subtensor_lift(fgraph, node):
     Handles the following unary ops:
     elemwise(x,...)[idx] -> elemwise(x[idx],...)
       when x,... are broadcasted scalar or not broadcasted at all
-    rebroadcast(x)[idx] => rebroadcast(x[idx])
+    Unbroadcast(x)[idx] => Unbroadcast(x[idx])
 
     """
     if isinstance(node.op, Subtensor):
@@ -430,34 +430,34 @@ def local_subtensor_lift(fgraph, node):
                 copy_stack_trace([node.outputs[0], node.inputs[0]], ret)
                 return [ret]
 
-        if isinstance(u.owner.op, Rebroadcast):
-            # make sure that Rebroadcast has only 1 input
-            assert len(u.owner.inputs) == 1
-
+        if isinstance(u.owner.op, Unbroadcast):
             # Subtensor might reduce dim., adapt broadcast pattern accordingly
-            new_axis = []
+            old_axes = u.owner.op.axes
+            new_axes = []
 
             # loop through indices being subtensor-ed
             # i indexes broadcastable pattern before subtensor
             # j indexes broadcastable pattern after subtensor
             j = 0
             for (i, x) in enumerate(node.op.idx_list):
-                # if its not a slice, it will reduce the dimension, should
+                # if it is not a slice, it will reduce the dimension, should
                 # not appear in the broascastable dimensions
                 if isinstance(x, slice):
-                    new_axis += [(j, u.broadcastable[i])]
+                    if i in old_axes:
+                        new_axes.append(j)
                     j += 1
             # now keep the broadcastable pattern of all
             # items not appearing in subtensor list
             for i in range(len(node.op.idx_list), len(u.broadcastable)):
-                new_axis += [(j, u.broadcastable[i])]
+                if i in old_axes:
+                    new_axes.append(j)
                 j += 1
 
             subt_x = node.op(u.owner.inputs[0], *node.inputs[1:])
             # Copy over previous output stacktrace
             copy_stack_trace(node.outputs[0], subt_x)
 
-            rbcast_subt_x = Rebroadcast(*new_axis)(subt_x)
+            rbcast_subt_x = unbroadcast(subt_x, *new_axes)
             # Copy over previous output stacktrace
             # and stacktrace from previous unary operation
             copy_stack_trace([node.outputs[0], node.inputs[0]], rbcast_subt_x)
@@ -533,14 +533,6 @@ def local_subtensor_merge(fgraph, node):
             # because of either of the two original slicing operations
             orig_out = node.outputs[0]
             copy_stack_trace([orig_out, node.inputs[0]], out)
-
-            # Restore original broadcastable dimensions that `subtens()` may
-            # have been unable to infer again
-            if not orig_out.type.is_super(out.type):
-                assert out.dtype == orig_out.dtype
-                assert out.ndim == orig_out.ndim
-                out = patternbroadcast(out, orig_out.broadcastable)
-                copy_stack_trace([orig_out, node.inputs[0]], out)
             return [out]
 
 
@@ -658,11 +650,6 @@ def local_subtensor_of_alloc(fgraph, node):
     rval = alloc(nw_val, *nw_dims)
     if not isinstance(rval, (list, tuple)):
         rval = [rval]
-    if not node.outputs[0].type.is_super(rval[0].type):
-        # It happen that the make_node() isn't able to infer the same pattern.
-        # We know it is safe, so fix that.
-        rval[0] = patternbroadcast(rval[0], node.outputs[0].broadcastable)
-
     return rval
 
 
@@ -766,7 +753,6 @@ def local_subtensor_make_vector(fgraph, node):
             values = list(map(int, list(idx.value)))
             ret = make_vector_op(*[x.owner.inputs[v] for v in values])
             copy_stack_trace(node.outputs[0], ret)
-            ret = patternbroadcast(ret, node.outputs[0].broadcastable)
             return [ret]
     elif isinstance(idx, slice):
         # The index is a slice.  If it's a constant slice, we can perform the
@@ -777,7 +763,6 @@ def local_subtensor_make_vector(fgraph, node):
             )[0]
             ret = make_vector_op(*x.owner.inputs[const_slice])
             copy_stack_trace(node.outputs, ret)
-            ret = patternbroadcast(ret, node.outputs[0].broadcastable)
             return [ret]
         except NotScalarConstantError:
             pass
